@@ -1,24 +1,23 @@
 /***************************************************************************
 
-  main.c
+	main.c
 
-  BZLIB2 Compression driver
+	(c) 2003-2004 Daniel Campos Fernández <danielcampos@netcourrier.com>
 
-  (c) 2003-2004 Daniel Campos Fernández <danielcampos@netcourrier.com>
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2, or (at your option)
+	any later version.
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
-  any later version.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+	You should have received a copy of the GNU General Public License
+	along with this program; if not, write to the Free Software
+	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+	MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -44,35 +43,35 @@
 #define P_POS  ((BZ2_STREAM*)stream)->info->pos
 
 
-typedef struct
-{
-	uint8_t mode;
-	uint8_t eof;
-	BZFILE  *bz;
-	FILE    *file;
-	int64_t pos;
-	
-	
-} handleInfo; 
+typedef 
+	struct
+	{
+		uint8_t mode;
+		uint8_t eof;
+		BZFILE  *bz;
+		FILE    *file;
+		int64_t pos;
+	} 
+	handleInfo; 
 
-typedef struct
-{
-    GB_STREAM_DESC *desc;
-    int _reserved;
-    handleInfo *info;
-
-} BZ2_STREAM;
+typedef 
+	struct
+	{
+		GB_STREAM_BASE base;
+		handleInfo *info;
+	} 
+	BZ2_STREAM;
 
 GB_INTERFACE GB EXPORT;
 COMPRESS_INTERFACE COMPRESSION EXPORT;
 
 static COMPRESS_DRIVER _driver;
 
-GB_STREAM_DESC BZStream = {
+GB_STREAM_DESC BZStream = 
+{
 	open: BZ_stream_open,
 	close: BZ_stream_close,
 	read: BZ_stream_read,
-	getchar: BZ_stream_getchar,
 	write: BZ_stream_write,
 	seek: BZ_stream_seek,
 	tell: BZ_stream_tell,
@@ -83,20 +82,20 @@ GB_STREAM_DESC BZStream = {
 
 /*****************************************************************************
 
-  The driver interface
+	The driver interface
 
 *****************************************************************************/
 
 /*********************************************************************************
- The Driver must provide this function:
+The Driver must provide this function:
 
- int max_compression(void)
+int max_compression(void)
 
- It must return the maximum compression level that user can
- assign, for instance gzip (in this moment) provides ten values, from zero to
- 9 , beeing 9 the best compression level (more compression),
- so the driver here returns '9'
- *********************************************************************************/
+It must return the maximum compression level that user can
+assign, for instance gzip (in this moment) provides ten values, from zero to
+9 , beeing 9 the best compression level (more compression),
+so the driver here returns '9'
+*********************************************************************************/
 
 static int BZ2_max_compression(void)
 {
@@ -104,52 +103,52 @@ static int BZ2_max_compression(void)
 }
 
 /*********************************************************************************
- The Driver must provide this function:
+The Driver must provide this function:
 
- int min_compression(void)
+int min_compression(void)
 
- It must return the minimum compression level that user can
- assign, for instance gzip (in this moment) provides ten values, from zero to
- 9 , beeing 0 the worst compression level (no compression at all),
- so the driver here returns '0'
- *********************************************************************************/
+It must return the minimum compression level that user can
+assign, for instance gzip (in this moment) provides ten values, from zero to
+9 , beeing 0 the worst compression level (no compression at all),
+so the driver here returns '0'
+*********************************************************************************/
 static int BZ2_min_compression(void)
 {
 	return 1;
 }
 /*********************************************************************************
- The Driver must provide this function:
+The Driver must provide this function:
 
- int default_compression(void)
+int default_compression(void)
 
- It must return the default compression level, that is, the value to send when
- Compression object methods are called without the optional parameter 'Level'.
- In this gzip wrapper, for example, this value is -1
- *********************************************************************************/
+It must return the default compression level, that is, the value to send when
+Compression object methods are called without the optional parameter 'Level'.
+In this gzip wrapper, for example, this value is -1
+*********************************************************************************/
 static int BZ2_default_compression(void)
 {
 	return 6;
 }
 /*********************************************************************************
- The Driver must provide this function:
+The Driver must provide this function:
 
- void c_String(char **target,unsigned long *lent,char *source,unsigned long len,int level)
+void c_String(char **target,unsigned long *lent,char *source,unsigned long len,int level)
 
- It is called to compress a String and return it compressed. The object will
- pass the following values:
+It is called to compress a String and return it compressed. The object will
+pass the following values:
 
- target = NULL
- lent   = 0
- source = a pointer to the original string
- len    = length of the original string
- level  = compression level
+target = NULL
+lent   = 0
+source = a pointer to the original string
+len    = length of the original string
+level  = compression level
 
- You will never receive a zero lenght string, nor a erroneus 'level' value
+You will never receive a zero lenght string, nor a erroneus 'level' value
 
- The function must store the compressed string in 'target', and its length in 'lent',
- or NULL and zero if it fails by any reason
+The function must store the compressed string in 'target', and its length in 'lent',
+or NULL and zero if it fails by any reason
 
- *********************************************************************************/
+*********************************************************************************/
 static void BZ2_c_String(char **target,unsigned long *lent,char *source,unsigned long len,int level)
 {
 	*target=NULL;
@@ -167,24 +166,24 @@ static void BZ2_c_String(char **target,unsigned long *lent,char *source,unsigned
 
 }
 /*********************************************************************************
- The Driver must provide this function:
+The Driver must provide this function:
 
- void u_String(char **target,unsigned long *lent,char *source,unsigned long len,int level)
+void u_String(char **target,unsigned long *lent,char *source,unsigned long len,int level)
 
- It is called to decompress a String and return it decompressed. The object will
- pass the following values:
+It is called to decompress a String and return it decompressed. The object will
+pass the following values:
 
- target = NULL
- lent   = 0
- source = a pointer to the original string
- len    = length of the original string
- level  = compression level
+target = NULL
+lent   = 0
+source = a pointer to the original string
+len    = length of the original string
+level  = compression level
 
-  You will never receive a zero lenght string, nor a erroneus 'level' value
+	You will never receive a zero lenght string, nor a erroneus 'level' value
 
- The function must store the decompressed string in 'target', and its length in 'lent',
- or NULL and zero if it fails by any reason
- *********************************************************************************/
+The function must store the decompressed string in 'target', and its length in 'lent',
+or NULL and zero if it fails by any reason
+*********************************************************************************/
 static void BZ2_u_String(char **target,unsigned long *lent,char *source,unsigned long len)
 {
 	int myok=BZ_OUTBUFF_FULL;
@@ -234,19 +233,19 @@ static void BZ2_u_String(char **target,unsigned long *lent,char *source,unsigned
 
 }
 /*********************************************************************************
- The Driver must provide this function:
+The Driver must provide this function:
 
 static void c_File(char *source,char *target,int level)
 
- It is called to compress a file. The object will pass the following values:
+It is called to compress a file. The object will pass the following values:
 
 
- source = path of the file to be compressed
- target = path of the new compressed file to create
- level  = compression level
+source = path of the file to be compressed
+target = path of the new compressed file to create
+level  = compression level
 
-  You will never receive a erroneus 'level' value
- *********************************************************************************/
+	You will never receive a erroneus 'level' value
+*********************************************************************************/
 static void BZ2_c_File(char *source,char *target,int level)
 {
 	FILE *src;
@@ -307,19 +306,19 @@ static void BZ2_c_File(char *source,char *target,int level)
 }
 
 /*********************************************************************************
- The Driver must provide this function:
+The Driver must provide this function:
 
 static void u_File(char *source,char *target,int level)
 
- It is called to decompress a file. The object will pass the following values:
+It is called to decompress a file. The object will pass the following values:
 
 
- source = path of the file to be decompressed
- target = path of the new decompressed file to create
- level  = compression level
+source = path of the file to be decompressed
+target = path of the new decompressed file to create
+level  = compression level
 
-  You will never receive a erroneus 'level' value
- *********************************************************************************/
+	You will never receive a erroneus 'level' value
+*********************************************************************************/
 static void BZ2_u_File(char *source,char *target)
 {
 	BZFILE *src;
@@ -379,16 +378,16 @@ static void BZ2_u_File(char *source,char *target)
 	fclose(dst);
 }
 
-static void BZ2_c_Open(char *path,int level,GB_STREAM *stream)
+static void BZ2_c_Open(char *path,int level,BZ2_STREAM *stream)
 {
 	int bzerror;
 
-	GB.Alloc( POINTER(&((BZ2_STREAM*)stream)->info),sizeof(handleInfo) );
+	GB.Alloc(POINTER(&stream->info), sizeof(handleInfo) );
 
 	P_FILE=fopen(path,"wb");
 	if (!P_FILE)
 	{
-		GB.Free( POINTER(&((BZ2_STREAM*)stream)->info) );
+		GB.Free(POINTER(&stream->info));
 		GB.Error("Unable to open file");
 		return;
 	}
@@ -407,8 +406,7 @@ static void BZ2_c_Open(char *path,int level,GB_STREAM *stream)
 
 	P_EOF=0;
 	P_POS=0;
-	stream->desc=&BZStream;
-	
+	stream->base.desc = &BZStream;
 }
 
 static void BZ2_u_Open(char *path,GB_STREAM *stream)
@@ -442,7 +440,7 @@ static void BZ2_u_Open(char *path,GB_STREAM *stream)
 }
 
 /*************************************************************************
- Stream related stuff
+Stream related stuff
 **************************************************************************/
 /* not allowed stream methods */
 static int BZ_stream_lof(GB_STREAM *stream, int64_t *len){return -1;}
@@ -464,7 +462,7 @@ static int BZ_stream_close(GB_STREAM *stream)
 {
 	int bzerror;
 
-	if (P_MODE==MODE_WRITE)
+	if (P_MODE == MODE_WRITE)
 		BZ2_bzWriteClose(&bzerror,P_BZ,0,NULL,NULL);
 	else
 		BZ2_bzReadClose(&bzerror,P_BZ);
@@ -482,13 +480,13 @@ static int BZ_stream_write(GB_STREAM *stream, char *buffer, int len)
 
 	if (P_MODE==MODE_READ) return -1;
 	BZ2_bzWrite ( &bzerror,P_BZ, (void*)buffer, len);
-	if (!bzerror) { P_POS+=len; return 0; }
+	if (!bzerror) { P_POS += len; return len; }
 
-	BZ2_bzWriteClose (&bzerror,P_BZ,0,NULL,NULL);
-	fclose(P_FILE);
+	//BZ2_bzWriteClose (&bzerror,P_BZ,0,NULL,NULL);
+	//fclose(P_FILE);
 
-	GB.Free( POINTER(&((BZ2_STREAM*)stream)->info) );
-	stream->desc=NULL;
+	//GB.Free( POINTER(&((BZ2_STREAM*)stream)->info) );
+	//stream->desc=NULL;
 	return -1;
 }
 
@@ -502,81 +500,77 @@ static int BZ_stream_read(GB_STREAM *stream, char *buffer, int len)
 	int bzerror;
 	int len2;
 
-	if (P_MODE==MODE_WRITE) return -1;
+	if (P_MODE == MODE_WRITE) 
+		return -1;
 
-	len2=BZ2_bzRead ( &bzerror,P_BZ,(void*)buffer,len);
+	len2 = BZ2_bzRead (&bzerror, P_BZ, (void*)buffer, len);
+	
 	if (!bzerror)
 	{
-		P_POS+=len2;
-		return 0;
+		P_POS += len2;
+		return len2;
 	}
 	else
 	{
-		if ((len2==len) && (bzerror==BZ_STREAM_END))
+		if ((len2 == len) && (bzerror == BZ_STREAM_END))
 		{
-			P_POS+=len2;
-			P_EOF=1;
-			return 0;
+			P_POS += len2;
+			P_EOF = 1;
+			return len2;
 		}
 	}
 
-	BZ2_bzReadClose (&bzerror,P_BZ);
-	fclose(P_FILE);
-
-	GB.Free( POINTER(&((BZ2_STREAM*)stream)->info) );
-	stream->desc=NULL;
+	//BZ2_bzReadClose (&bzerror,P_BZ);
+	//fclose(P_FILE);
+	//GB.Free( POINTER(&((BZ2_STREAM*)stream)->info) );
+	//stream->desc=NULL;
 	return -1;
 }
 
-static int BZ_stream_getchar(GB_STREAM *stream, char *buffer)
-{
-	return BZ_stream_read(stream,buffer,1);
-}
-
 
 /****************************************************************************
- This array of functions defines what functions the compression component
- must call to perform its actions
- ****************************************************************************/
+This array of functions defines what functions the compression component
+must call to perform its actions
+****************************************************************************/
 static COMPRESS_DRIVER _driver =
 {
-    "bzlib2",
+		"bzlib2",
 
-    (void*)BZ2_max_compression,
-    (void*)BZ2_min_compression,
-    (void*)BZ2_default_compression,
+		(void*)BZ2_max_compression,
+		(void*)BZ2_min_compression,
+		(void*)BZ2_default_compression,
 
-    {
-    	(void*)BZ2_c_String,
+		{
+			(void*)BZ2_c_String,
 	(void*)BZ2_c_File,
 	(void*)BZ2_c_Open,
 	(void*)BZ_stream_close,
-    },
+		},
 
-    {
-    	(void*)BZ2_u_String,
+		{
+			(void*)BZ2_u_String,
 	(void*)BZ2_u_File,
 	(void*)BZ2_u_Open,
 	(void*)BZ_stream_close
-    }
+		}
 
 };
 /*****************************************************************************
 
-  The component entry and exit functions.
+	The component entry and exit functions.
 
 *****************************************************************************/
 
 int EXPORT GB_INIT(void)
 {
-  /*************************************************************************
-   When this component is loaded by Gambas runtime, it 'informs' to the
-   compression component what functions must it call to perform its actions
-   *************************************************************************/
-  GB.GetInterface("gb.compress", COMPRESS_INTERFACE_VERSION, &COMPRESSION);
-  COMPRESSION.Register(&_driver);
+	/*************************************************************************
+	When this component is loaded by Gambas runtime, it 'informs' to the
+	compression component what functions must it call to perform its actions
+	*************************************************************************/
+	GB.GetInterface("gb.compress", COMPRESS_INTERFACE_VERSION, &COMPRESSION);
+	COMPRESSION.Register(&_driver);
 
-  return FALSE;
+	return 0;
 }
 
 void EXPORT GB_EXIT()
