@@ -4,15 +4,9 @@
 
   (c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
 
-  Gtkmae "GTK+ made easy" classes
-
-  Realizado para la Junta de Extremadura.
-  Consejería de Educación Ciencia y Tecnología.
-  Proyecto gnuLinEx
-
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -22,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -38,6 +33,15 @@
 #endif
 #include <gtk/gtk.h>
 
+#ifdef GTK3
+
+#ifdef GDK_WINDOWING_X11
+#include <gtk/gtkx.h>
+#endif
+#include <gdk/gdkkeysyms-compat.h>
+
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -45,6 +49,8 @@
 #include <stdint.h>
 #include <inttypes.h>
 
+#include "gb_common.h"
+
 #include "gb.form.properties.h"
 #include "gb.form.const.h"
 
@@ -54,8 +60,8 @@
 #include "gcontrol.h"
 #include "gcontainer.h"
 #include "gtools.h"
-#include "gmemory.h"
 
+#include "main.h"
 //#define GTK_DEBUG_SIGNALS
 //#define GTK_DEBUG_OBJECTS
 
@@ -86,8 +92,7 @@ enum
 	Type_gListView    = 0x1010,
 	Type_gColumnView  = 0x1012,
 	Type_gTreeView    = 0x1014,
-	Type_gIconView    = 0x1015,
-	Type_gGridView    = 0x1016,
+	Type_gGLArea      = 0x0018,
 };
 
 enum
@@ -105,9 +110,55 @@ enum
  gEvent_FocusOut,
  gEvent_Enter,
  gEvent_Leave,
+ gEvent_DragMove,
+ gEvent_Drop
 };
 
 typedef
 	unsigned char uchar;
 
+#ifdef GTK3
+	#define ON_DRAW_BEFORE(_widget, _this, _gtk, _gtk3) g_signal_connect(G_OBJECT(_widget), "draw", G_CALLBACK(_gtk3), (gpointer)_this)
+	#define ON_DRAW(_widget, _this, _gtk, _gtk3) g_signal_connect_after(G_OBJECT(_widget), "draw", G_CALLBACK(_gtk3), (gpointer)_this)
+#else
+	#define ON_DRAW_BEFORE(_widget, _this, _gtk, _gtk3) g_signal_connect(G_OBJECT(_widget), "expose-event", G_CALLBACK(_gtk), (gpointer)_this)
+	#define ON_DRAW(_widget, _this, _gtk, _gtk3) g_signal_connect_after(G_OBJECT(_widget), "expose-event", G_CALLBACK(_gtk), (gpointer)_this)
+#endif
+
+#ifdef GTK3
+
+#define STATE_T GtkStateFlags
+#define STYLE_T GtkStyleContext
+
+#define STATE_NORMAL GTK_STATE_FLAG_NORMAL
+#define STATE_ACTIVE GTK_STATE_FLAG_ACTIVE
+#define STATE_INSENSITIVE GTK_STATE_FLAG_INSENSITIVE
+#define STATE_PRELIGHT GTK_STATE_FLAG_PRELIGHT
+#define STATE_SELECTED GTK_STATE_FLAG_SELECTED
+#define STATE_FOCUSED GTK_STATE_FLAG_FOCUSED
+
+#define gtk_hbox_new(_homogeneous, _spacing) gtk_box_new(GTK_ORIENTATION_HORIZONTAL, _spacing)
+#define gtk_vbox_new(_homogeneous, _spacing) gtk_box_new(GTK_ORIENTATION_VERTICAL, _spacing)
+
+#define gtk_hscale_new(_adj) gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, _adj)
+#define gtk_vscale_new(_adj) gtk_scale_new(GTK_ORIENTATION_VERTICAL, _adj)
+
+#define gtk_hscrollbar_new(_adj) gtk_scrollbar_new(GTK_ORIENTATION_HORIZONTAL, _adj)
+#define gtk_vscrollbar_new(_adj) gtk_scrollbar_new(GTK_ORIENTATION_VERTICAL, _adj)
+
+#define gtk_modify_font gtk_override_font
+
+#else
+
+#define STATE_T GtkStateType
+#define STYLE_T GtkStyle
+
+#define STATE_NORMAL GTK_STATE_NORMAL
+#define STATE_ACTIVE GTK_STATE_ACTIVE
+#define STATE_INSENSITIVE GTK_STATE_INSENSITIVE
+#define STATE_PRELIGHT GTK_STATE_PRELIGHT
+#define STATE_SELECTED GTK_STATE_SELECTED
+
+#endif
+
 #endif
