@@ -2,19 +2,11 @@
 
   CPdfDocument.cpp
 
-  gb.pdf Component
-
   (C) 2005-2007 Daniel Campos Fernández <dcamposf@gmail.com>
 
-
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
-  any later version.
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -24,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -51,33 +44,110 @@
 #include <Outline.h>
 #include <Link.h>
 #include <Gfx.h>
+#include <GlobalParams.h>
+#include <UnicodeMap.h>
 
-/*****************************************************************************
+#if POPPLER_VERSION_0_72
+#define getCString c_str
+#endif
+
+/***************************************************************************/
+
+static CPDFRECT *create_rect(void)
+{
+	return (CPDFRECT *)GB.New(GB.FindClass("PdfRect"), NULL, NULL);
+}
+
+BEGIN_PROPERTY(PdfRect_X)
+
+	GB.ReturnFloat(THIS_RECT->x);
+
+END_PROPERTY
+
+BEGIN_PROPERTY(PdfRect_Y)
+
+	GB.ReturnFloat(THIS_RECT->y);
+
+END_PROPERTY
+
+BEGIN_PROPERTY(PdfRect_Width)
+
+	GB.ReturnFloat(THIS_RECT->w);
+
+END_PROPERTY
+
+BEGIN_PROPERTY(PdfRect_Height)
+
+	GB.ReturnFloat(THIS_RECT->h);
+
+END_PROPERTY
+
+BEGIN_PROPERTY(PdfRect_Right)
+
+	GB.ReturnFloat(THIS_RECT->x + THIS_RECT->w);
+
+END_PROPERTY
+
+BEGIN_PROPERTY(PdfRect_Bottom)
+
+	GB.ReturnFloat(THIS_RECT->y + THIS_RECT->h);
+
+END_PROPERTY
+
+
+
+/****************************************************************************
 
  Translations from Poppler universe to Gambas universe
 
-******************************************************************************/
-typedef struct
+****************************************************************************/
+
+static void return_unicode_string(const Unicode *unicode, int len)
 {
-	int32_t x0;
-	int32_t y0;
-	int32_t x1;
-	int32_t y1;
-} FoundRect;
+	static UnicodeMap *uMap = NULL;
+	
+	GooString gstr;
+	char buf[8]; /* 8 is enough for mapping an unicode char to a string */
+	int i, n;
+
+	if (uMap == NULL) 
+	{
+		GooString *enc = new GooString("UTF-8");
+		uMap = globalParams->getUnicodeMap(enc);
+		uMap->incRefCnt();
+		delete enc;
+	}
+		
+	for (i = 0; i < len; ++i) {
+		n = uMap->mapUnicode(unicode[i], buf, sizeof(buf));
+		gstr.append(buf, n);
+	}
+
+	GB.ReturnNewZeroString(gstr.getCString());
+}
+
 
 static void aux_return_string_info(void *_object, const char *key)
 {
 	Object obj;
 	Object dst;
-	GooString *goo_value;
+	const_GooString *goo_value;
 	Dict *info_dict;
 	char *tmpstr;
 
+	#if POPPLER_VERSION_0_58
+	obj = THIS->doc->getDocInfo ();
+	#else
 	THIS->doc->getDocInfo (&obj);
+	#endif
 	if (!obj.isDict()) { GB.ReturnNewZeroString(""); return; }
 		
 	info_dict=obj.getDict();
+	#if POPPLER_VERSION_0_58
+	dst = info_dict->lookup ((char *)key);
+	#else
 	info_dict->lookup ((char *)key, &dst);
+	#endif
 	if (!dst.isString ()) { GB.ReturnNewZeroString(""); }
 	else {
 		goo_value = dst.getString();
@@ -90,8 +160,10 @@ static void aux_return_string_info(void *_object, const char *key)
 		else
 			GB.ReturnNewString(goo_value->getCString(),goo_value->getLength());		
 	}
+	#if ! POPPLER_VERSION_0_58
 	dst.free();
 	obj.free();		
+	#endif
 }
 
 static void aux_return_date_info(void *_object, const char *key)
@@ -101,39 +173,44 @@ static void aux_return_date_info(void *_object, const char *key)
 	GB_DATE ret;
 	Object obj;
 	Object dst;
-	GooString *goo;
+	const_GooString *goo;
 	Dict *info_dict;
 	char *datestr=NULL,*tofree=NULL;
 	int nnum;
 
+	GB.ReturnDate(NULL);
+	
+	#if POPPLER_VERSION_0_58
+	obj = THIS->doc->getDocInfo ();
+	#else
 	THIS->doc->getDocInfo (&obj);
-	if (!obj.isDict()) { GB.ReturnNull(); return; }
+	#endif
+	if (!obj.isDict()) return;
 
 	info_dict=obj.getDict();
+	#if POPPLER_VERSION_0_58
+	dst = info_dict->lookup ((char *)key);
+	#else
 	info_dict->lookup ((char *)key, &dst);
-	if (!dst.isString ()) { GB.ReturnNull(); }
-	else {
+	#endif
+	if (dst.isString ())
+	{
 		goo = dst.getString();
 		if (goo->hasUnicodeMarker())
 			GB.ConvString (&datestr,goo->getCString()+2,goo->getLength()-2,"UTF-16BE","UTF-8");
 		else
 		{
-			GB.NewString(&datestr,goo->getCString(),goo->getLength());
+			datestr = GB.NewString(goo->getCString(),goo->getLength());
 			tofree=datestr;		
 		}
 
-		if (!datestr) { GB.ReturnNull(); }
-		else
+		if (datestr)
 		{
 			if (datestr[0] == 'D' && datestr[1] == ':') datestr += 2;
-			nnum=sscanf(datestr, "%4hd%2hd%2hd%2hd%2hd%2hd",&ds.year, &ds.month, \
-			                                          &ds.day, &ds.hour, &ds.min, &ds.sec);	
-			if (nnum != 6) { GB.ReturnNull(); }
-			else
+			nnum=sscanf(datestr, "%4d%2d%2d%2d%2d%2d",&ds.year, &ds.month, &ds.day, &ds.hour, &ds.min, &ds.sec);
+			if (nnum == 6)
 			{
-				if (GB.MakeDate(&ds,&ret))
-					GB.ReturnNull();
-				else
+				if (!GB.MakeDate(&ds,&ret))
 					GB.ReturnDate(&ret);
 			}		
 		}
@@ -141,24 +218,17 @@ static void aux_return_date_info(void *_object, const char *key)
 	}
 
 	if (tofree) GB.FreeString(&tofree);
+	#if ! POPPLER_VERSION_0_58
 	dst.free();
 	obj.free();
+	#endif
 }
 
-static void aux_return_unicode_string(Unicode *uni, int32_t  len)
-{
-	int32_t bc;
-	char *ret=NULL;
-
-	for (bc=0; bc<len; bc++)
-		GB.AddString(&ret,(const char*)&uni[bc],0);
-
-	GB.ReturnNewString(ret,0);
-	GB.FreeString(&ret);
-}
-
-static LinkDest *get_dest(LinkAction *act)
+static const_LinkDest *get_dest(const_LinkAction *act)
 {
+	if (!act)
+		return 0;
+	
 	switch (act->getKind())
 	{
 		case actionGoTo: return ((LinkGoTo*)act)->getDest();
@@ -167,12 +237,12 @@ static LinkDest *get_dest(LinkAction *act)
 	}
 }
 
-static uint32_t aux_get_page_from_action(void *_object, LinkAction *act)
+static uint32_t aux_get_page_from_action(void *_object, const_LinkAction *act)
 {
 	Ref pref;       
-	LinkDest *dest = get_dest(act);
+	const_LinkDest *dest = get_dest(act);
 	#if POPPLER_VERSION_0_6
-	GooString *name;
+	const_GooString *name;
 	#else
 	UGooString *name;
 	#endif
@@ -185,8 +255,15 @@ static uint32_t aux_get_page_from_action(void *_object, LinkAction *act)
 		if (act->getKind () == actionGoTo)
 		{
 			name = ((LinkGoTo*)act)->getNamedDest();
-			if (name)
+			if (name) {
+			#if POPPLER_VERSION_0_64
 				dest = THIS->doc->findDest(name);
+			#elif POPPLER_VERSION_0_6
+				dest = THIS->doc->findDest((GooString *) name);
+			#else
+				dest = THIS->doc->findDest((UGooString *) name);
+			#endif
+			}
 		}
 	}
 
@@ -202,38 +279,33 @@ static uint32_t aux_get_page_from_action(void *_object, LinkAction *act)
 		return dest->getPageNum();
 }
 
-static void aux_get_dimensions_from_action(LinkAction *act,int *left, int32_t *right, int32_t *top, int32_t *bottom)
+
+static void aux_get_dimensions_from_action(const_LinkAction *act, CPDFRECT *rect)
 {
-	LinkDest *dest = get_dest(act);
+	const_LinkDest *dest = get_dest(act);
 	if (!dest)
-	{
-		if (left)   *left=0;
-		if (right)  *right=0;
-		if (top)    *top=0;
-		if (bottom) *bottom=0;
 		return;
-	}
 	
-	if (left)   *left=(int32_t)dest->getLeft();
-	if (right)  *right=(int32_t)dest->getRight();
-	if (top)    *top=(int32_t)dest->getTop();
-	if (bottom) *bottom=(int32_t)dest->getBottom();
+	rect->x = dest->getLeft();
+	rect->w = dest->getRight() - rect->x;
+	rect->y = dest->getTop();
+	rect->h = dest->getBottom() - rect->y;
 }
 
-static double aux_get_zoom_from_action(LinkAction *act)
+static double aux_get_zoom_from_action(const_LinkAction *act)
 {
-	LinkDest *dest = get_dest(act);
+	const_LinkDest *dest = get_dest(act);
 	if (dest)
 		return dest->getZoom();
 	else
 		return 1;
 }
 
-static char* aux_get_target_from_action(LinkAction *act)
+static char* aux_get_target_from_action(const_LinkAction *act)
 {
 	char *vl=NULL;
 	char *uni=NULL;	
-	GooString *tmp=NULL;
+	const_GooString *tmp=NULL;
 
 	switch (act->getKind())
 	{
@@ -265,10 +337,10 @@ static char* aux_get_target_from_action(LinkAction *act)
 	if (tmp->hasUnicodeMarker())
 	{
 			GB.ConvString (&uni,tmp->getCString()+2,tmp->getLength()-2,"UTF-16BE","UTF-8");
-			GB.AddString (&vl,uni,0);	
+			vl = GB.AddString(vl, uni, 0);	
 	}	
 	else
-			GB.AddString(&vl,tmp->getCString(),tmp->getLength());
+			vl = GB.AddString(vl,tmp->getCString(),tmp->getLength());
 	
 
 	return vl;
@@ -298,7 +370,7 @@ static void free_all(void *_object)
 
 	if (THIS->buf)
 	{
-		GB.ReleaseFile(&THIS->buf,THIS->len);
+		GB.ReleaseFile(THIS->buf,THIS->len);
 		THIS->buf=NULL;
 	}
 
@@ -384,13 +456,17 @@ int32_t open_document (void *_object, char *sfile, int32_t lfile)
 
 	if ( GB.LoadFile(sfile,lfile,&buf,&len) ) return -1;
 
+	#if POPPLER_VERSION_0_58
+	stream=new MemStream(buf,0,(Guint)len,std::move(obj));
+	#else
 	obj.initNull();
 	stream=new MemStream(buf,0,(Guint)len,&obj);
+	#endif
 	test=new PDFDoc (stream,0,0);
 
 	if (!test->isOk())
 	{
-		GB.ReleaseFile(&buf,len);
+		GB.ReleaseFile(buf,len);
 		ret=test->getErrorCode();
 		delete test;
 		test=NULL;
@@ -405,10 +481,14 @@ int32_t open_document (void *_object, char *sfile, int32_t lfile)
 	THIS->len=len;
 
 	white[0] = 0xFF; white[1] = 0xFF; white[2] = 0xFF;
-	THIS->dev=new SplashOutputDev(splashModeRGB8, 3, gFalse, white);
-		
-	THIS->dev->startDoc(THIS->doc->getXRef ());
+	THIS->dev=new SplashOutputDev(splashModeRGB8, 3, false, white);
 
+	#if POPPLER_VERSION_0_20
+	THIS->dev->startDoc(THIS->doc);
+	#else
+	THIS->dev->startDoc(THIS->doc->getXRef ());
+	#endif
+	
 	outline=THIS->doc->getOutline();
 	if (outline) THIS->index=outline->getItems();
 	
@@ -462,7 +542,6 @@ BEGIN_METHOD(PDFDOCUMENT_get,GB_INTEGER index;)
 	if (!THIS->doc || (VARG(index)<1) || ( VARG(index)>THIS->doc->getNumPages() ) )
 	{
 		GB.Error("Invalid page number");
-		GB.ReturnNull();
 		return;
 	}
 
@@ -652,7 +731,7 @@ BEGIN_PROPERTY(PDFDOCUMENT_index)
 END_PROPERTY
 
 BEGIN_PROPERTY(PDFINDEX_count)
-	
+
 	GB.ReturnInteger(THIS->index->getLength());
 
 END_PROPERTY
@@ -685,7 +764,7 @@ BEGIN_PROPERTY(PDFINDEX_title)
 	OutlineItem *item;
 
 	item = (OutlineItem *)THIS->index->get (THIS->currindex);
-	aux_return_unicode_string(item->getTitle(),item->getTitleLength());
+	return_unicode_string(item->getTitle(), item->getTitleLength());
 
 END_PROPERTY
 
@@ -779,21 +858,40 @@ END_METHOD
 
 ******************************************************************************/
 
+static int get_rotation(void *_object)
+{
+	return (THIS->rotation + THIS->page->getRotate() + 720) % 360;
+}
+
+static void get_page_size(void *_object, int *w, int *h)
+{
+	int rotation = get_rotation(THIS);
+
+	if (rotation == 90 || rotation == 270)
+	{
+		if (w) *w =  (int)(THIS->page->getMediaHeight() * THIS->scale);
+		if (h) *h = (int)(THIS->page->getMediaWidth() * THIS->scale);
+	}
+	else
+	{
+		if (w) *w = (int)(THIS->page->getMediaWidth() * THIS->scale);
+		if (h) *h =  (int)(THIS->page->getMediaHeight() * THIS->scale);
+	}
+}
+
 BEGIN_PROPERTY (PDFPAGE_width)
 
-	if ( (THIS->rotation==90) || (THIS->rotation==270) ) 
-		GB.ReturnInteger((int32_t)(THIS->page->getMediaHeight()*THIS->scale));
-	else
-		GB.ReturnInteger((int32_t)(THIS->page->getMediaWidth()*THIS->scale));
+	int w;
+	get_page_size(THIS, &w, NULL);
+	GB.ReturnInteger(w);
 
 END_PROPERTY
 
 BEGIN_PROPERTY (PDFPAGE_height)
 
-	if ( (THIS->rotation==90) || (THIS->rotation==270) )
-		GB.ReturnInteger((int32_t)(THIS->page->getMediaWidth()*THIS->scale));
-	else	
-		GB.ReturnInteger((int32_t)(THIS->page->getMediaHeight()*THIS->scale));
+	int h;
+	get_page_size(THIS, NULL, &h);
+	GB.ReturnInteger(h);
 
 END_PROPERTY
 
@@ -802,19 +900,10 @@ static uint32_t *get_page_data(CPDFDOCUMENT *_object, int32_t x, int32_t y, int3
 	SplashBitmap *map;
 	uint32_t *data;
 	int32_t w, h;
-	int32_t rw;
-	int32_t rh;
+	int rw;
+	int rh;
 
-	if ( (THIS->rotation==90) || (THIS->rotation==270) )
-	{
-		rh=(int32_t)(THIS->page->getMediaWidth()*THIS->scale);
-		rw=(int32_t)(THIS->page->getMediaHeight()*THIS->scale);
-	}
-	else
-	{
-		rw=(int32_t)(THIS->page->getMediaWidth()*THIS->scale);
-		rh=(int32_t)(THIS->page->getMediaHeight()*THIS->scale);
-	}
+	get_page_size(THIS, &rw, &rh);
 
 	w = *width;
 	h = *height;
@@ -833,14 +922,23 @@ static uint32_t *get_page_data(CPDFDOCUMENT *_object, int32_t x, int32_t y, int3
 
 	if ( (w<0) || (h<0) ) return NULL;
 
+	#if POPPLER_VERSION_0_20
 	THIS->page->displaySlice(THIS->dev,72.0*scale,72.0*scale,
 			   rotation,
-			   gFalse,
-			   gTrue,
+			   false,
+			   true,
 			   x,y,w,h,
-			   gFalse,
+			   false);
+	#else
+	THIS->page->displaySlice(THIS->dev,72.0*scale,72.0*scale,
+			   rotation,
+			   false,
+			   true,
+			   x,y,w,h,
+			   false,
 			   THIS->doc->getCatalog ());
-
+	#endif
+	
 	map=THIS->dev->getBitmap();
 	
 	data=(uint32_t*)map->getDataPtr();
@@ -854,7 +952,6 @@ static uint32_t *get_page_data(CPDFDOCUMENT *_object, int32_t x, int32_t y, int3
 
 BEGIN_METHOD(PDFPAGE_image, GB_INTEGER x; GB_INTEGER y; GB_INTEGER w; GB_INTEGER h)
 
-	GB_IMAGE img = NULL;
 	uint32_t *data;
 	int32_t x,y, w, h;
 
@@ -865,8 +962,10 @@ BEGIN_METHOD(PDFPAGE_image, GB_INTEGER x; GB_INTEGER y; GB_INTEGER w; GB_INTEGER
 
 	data = get_page_data(THIS, x, y, &w, &h, THIS->scale, THIS->rotation);
 	if (!data) { GB.ReturnNull(); return; }
-	GB.Image.Create(&img, data, w, h, GB_IMAGE_RGB);
-	GB.ReturnObject(img);
+	/*GB.Image.Create(&img, data, w, h, GB_IMAGE_RGB);
+	GB.ReturnObject(img);*/
+
+	GB.ReturnObject(IMAGE.Create(w, h, GB_IMAGE_RGB, (unsigned char *)data));
 
 END_METHOD
 
@@ -874,46 +973,14 @@ BEGIN_PROPERTY (PDFPAGE_property_image)
 
 	int32_t w=-1;
 	int32_t h=-1;
-	GB_IMAGE img = NULL;
 	uint32_t *data;
 
 	data = get_page_data(THIS, 0, 0, &w, &h, THIS->scale, THIS->rotation);
 	if (!data) { GB.ReturnNull(); return; }
-	GB.Image.Create(&img, data, w, h, GB_IMAGE_RGB);
-	GB.ReturnObject(img);
+	/*GB.Image.Create(&img, data, w, h, GB_IMAGE_RGB);
+	GB.ReturnObject(img);*/
 
-END_PROPERTY
-
-
-BEGIN_METHOD(PDFPAGE_picture, GB_INTEGER x; GB_INTEGER y; GB_INTEGER w; GB_INTEGER h)
-
-	GB_IMAGE img = NULL;
-	uint32_t *data;
-	int32_t x,y, w, h;
-
-	x = VARGOPT(x, 0);
-	y = VARGOPT(y, 0);
-	w = VARGOPT(w, -1);
-	h = VARGOPT(h, -1);
-
-	data = get_page_data(THIS, x, y, &w, &h, THIS->scale, THIS->rotation);
-	if (!data) { GB.ReturnNull(); return; }
-	GB.Picture.Create(&img, data, w, h, GB_IMAGE_RGB);
-	GB.ReturnObject(img);
-
-END_METHOD
-
-BEGIN_PROPERTY (PDFPAGE_property_picture)
-
-	int32_t w=-1;
-	int32_t h=-1;
-	GB_IMAGE img = NULL;
-	uint32_t *data;
-
-	data = get_page_data(THIS, 0, 0, &w, &h, THIS->scale, THIS->rotation);
-	if (!data) { GB.ReturnNull(); return; }
-	GB.Picture.Create(&img, data, w, h, GB_IMAGE_RGB);
-	GB.ReturnObject(img);
+	GB.ReturnObject(IMAGE.Create(w, h, GB_IMAGE_RGB, (unsigned char *)data));
 
 END_PROPERTY
 
@@ -929,10 +996,14 @@ BEGIN_METHOD(PDFPAGE_select, GB_INTEGER X; GB_INTEGER Y; GB_INTEGER W; GB_INTEGE
 	w = VARGOPT(W, (int32_t)THIS->page->getMediaWidth());
 	h = VARGOPT(H, (int32_t)THIS->page->getMediaHeight());
 
-	dev = new TextOutputDev (NULL, gTrue, gFalse, gFalse);
-	gfx = THIS->page->createGfx(dev,72.0,72.0,0,gFalse,gTrue,-1, -1, -1, -1, \
-                                gFalse,THIS->doc->getCatalog (),NULL, NULL, NULL, NULL);
-	
+	#if POPPLER_VERSION_0_20
+	dev = new TextOutputDev (NULL, true, 0, false, false);
+	gfx = THIS->page->createGfx(dev,72.0,72.0,0,false,true,-1, -1, -1, -1, false, NULL, NULL);
+	#else
+	dev = new TextOutputDev (NULL, true, false, false);
+	gfx = THIS->page->createGfx(dev,72.0,72.0,0,false,true,-1, -1, -1, -1, false,THIS->doc->getCatalog (),NULL, NULL, NULL, NULL);
+	#endif
+
 	THIS->page->display(gfx);
 	dev->endPage();
 
@@ -957,12 +1028,19 @@ END_METHOD
  Bookmarks of a PDF page
 
 ******************************************************************************/
+
 void aux_fill_links(void *_object)
 {
+	#if POPPLER_VERSION_0_20
+	THIS->links = new Links (THIS->page->getAnnots ());
+	#elif POPPLER_VERSION_0_17
+	THIS->links = new Links (THIS->page->getAnnots (THIS->doc->getCatalog()));
+	#else
 	Object obj;
 	
 	THIS->links = new Links (THIS->page->getAnnots (&obj),THIS->doc->getCatalog()->getBaseURI ());
 	obj.free();
+	#endif
 }
 
 BEGIN_PROPERTY (PDFPAGELINKS_count)
@@ -989,7 +1067,7 @@ BEGIN_METHOD (PDFPAGELINKS_get,GB_INTEGER ind;)
 		}
 	}
 
-	if (!pok) { GB.Error("Out of bounds"); GB.ReturnNull(); return; }
+	if (!pok) { GB.Error("Out of bounds"); return; }
 
 	THIS->lcurrent=VARG(ind);
 	THIS->action=THIS->links->getLink(THIS->lcurrent)->getAction();
@@ -1021,40 +1099,11 @@ BEGIN_PROPERTY (PDFPAGELINKDATA_uri)
 
 END_PROPERTY
 
-BEGIN_PROPERTY(PDFPAGELINKDATA_left)
+BEGIN_PROPERTY(PdfPageLinkData_Rect)
 
-	int32_t vl;
-
-	aux_get_dimensions_from_action(THIS->action,&vl, NULL, NULL, NULL);
-	GB.ReturnInteger(vl);
-
-END_PROPERTY
-
-BEGIN_PROPERTY(PDFPAGELINKDATA_right)
-
-	int32_t vl;
-
-	aux_get_dimensions_from_action(THIS->action,NULL,&vl, NULL, NULL);
-	GB.ReturnInteger(vl);
-
-END_PROPERTY
-
-BEGIN_PROPERTY(PDFPAGELINKDATA_top)
-
-	int32_t vl;
-
-	aux_get_dimensions_from_action(THIS->action,NULL,NULL,&vl, NULL);
-	GB.ReturnInteger(vl);
-
-END_PROPERTY
-
-BEGIN_PROPERTY(PDFPAGELINKDATA_bottom)
-
-	int32_t vl;
-
-	aux_get_dimensions_from_action(THIS->action,NULL, NULL, NULL,&vl);
-
-	GB.ReturnInteger(vl);
+	CPDFRECT *rect = create_rect();
+	aux_get_dimensions_from_action(THIS->action, rect);
+	GB.ReturnObject(rect);
 
 END_PROPERTY
 
@@ -1085,7 +1134,7 @@ BEGIN_PROPERTY(PDFPAGELINKDATA_check)
 
 END_PROPERTY
 
-void aux_get_link_dimensions(void *_object,int32_t *left, int32_t *top, int32_t *width, int32_t *height)
+static void aux_get_link_dimensions(void *_object, CPDFRECT *rect)
 {
 	double l,t,w,h;
 	double pw,ph;
@@ -1097,76 +1146,46 @@ void aux_get_link_dimensions(void *_object,int32_t *left, int32_t *top, int32_t
 	w=w-l;
 	h=h-t;	
 
-	switch (THIS->rotation)
+	switch (get_rotation(THIS))
 	{
 		case 0:
-			if (left) *left=(int32_t)(l*THIS->scale);
-			if (top) *top=(int32_t)((ph-t-h)*THIS->scale);
-			if (width) *width=(int32_t)(w*THIS->scale);
-			if (height) *height=(int32_t)(h*THIS->scale);
+			rect->x = (l*THIS->scale);
+			rect->y = ((ph-t-h)*THIS->scale);
+			rect->w = (w*THIS->scale);
+			rect->h = (h*THIS->scale);
 			break;
 	
 		case 90:
-			if (top) *top=(int32_t)(l*THIS->scale);
-			if (left) *left=(int32_t)(t*THIS->scale);
-			if (height) *height=(int32_t)(w*THIS->scale);
-			if (width) *width=(int32_t)(h*THIS->scale);
+			rect->y = (l*THIS->scale);
+			rect->x = (t*THIS->scale);
+			rect->h = (w*THIS->scale);
+			rect->w = (h*THIS->scale);
 			break;
 
 		case 180:
-			if (left) *left=(int32_t)((l-w)*THIS->scale);
-			if (top) *top=(int32_t)(t*THIS->scale);
-			if (width) *width=(int32_t)(w*THIS->scale);
-			if (height) *height=(int32_t)(h*THIS->scale);
+			rect->x = ((l-w)*THIS->scale);
+			rect->y = (t*THIS->scale);
+			rect->w = (w*THIS->scale);
+			rect->h = (h*THIS->scale);
 			break;
 
 		case 270:
-			if (top) *top=(int32_t)((pw-l-w)*THIS->scale);
-			if (left) *left=(int32_t)((ph-t-h)*THIS->scale);
-			if (height) *height=(int32_t)(w*THIS->scale);
-			if (width) *width=(int32_t)(h*THIS->scale);
+			rect->y = ((pw-l-w)*THIS->scale);
+			rect->x = ((ph-t-h)*THIS->scale);
+			rect->h = (w*THIS->scale);
+			rect->w = (h*THIS->scale);
 			break;
 	}
-
 }
 
-BEGIN_PROPERTY (PDFPAGELINK_width)
+BEGIN_PROPERTY(PdfPageLink_rect)
 
-	int32_t vl;
-	
-	aux_get_link_dimensions(_object,NULL,NULL,&vl,NULL);
-	GB.ReturnInteger(vl);
+	CPDFRECT *rect = create_rect();
+	aux_get_link_dimensions(THIS, rect);
+	GB.ReturnObject(rect);
 
 END_PROPERTY
 
-BEGIN_PROPERTY (PDFPAGELINK_height)
-
-	int32_t vl;
-	
-	aux_get_link_dimensions(_object,NULL,NULL,NULL,&vl);
-	GB.ReturnInteger(vl);
-
-
-END_PROPERTY
-
-BEGIN_PROPERTY (PDFPAGELINK_left)
-
-	int32_t vl;
-	
-	aux_get_link_dimensions(_object,&vl,NULL,NULL,NULL);
-	GB.ReturnInteger(vl);
-	
-
-END_PROPERTY
-
-BEGIN_PROPERTY (PDFPAGELINK_top)
-
-	int32_t vl;
-	
-	aux_get_link_dimensions(_object,NULL,&vl,NULL,NULL);
-	GB.ReturnInteger(vl);
-
-END_PROPERTY
 
 /*****************************************************************************
 
@@ -1179,15 +1198,18 @@ BEGIN_METHOD (PDFPAGE_find,GB_STRING Text; GB_BOOLEAN Sensitive;)
 	TextOutputDev *textdev;
 	double x0=0, y0=0;
 	double x1, y1;
-	FoundRect *el;	
+	CPDFFIND *el;	
 	Unicode *block=NULL;
-	int32_t nlen=0;
+	int nlen=0;
 	bool sensitive=false;
+	int count;
+	double x, y, w, h, wp, hp;
+	int rotation;
 
 	// TODO: Use UCS-4BE on big endian systems?
-	if (GB.ConvString ((char **)(void *)&block,STRING(Text),LENGTH(Text),"UTF-8","UCS-4LE"))
+	if (GB.ConvString ((char **)(void *)&block,STRING(Text),LENGTH(Text),"UTF-8",GB_SC_UNICODE))
 	{	
-		GB.Error("Invalid UTF-8 string"); 
+		GB.Error("Invalid UTF-8 string");
 		return;
 	}
 
@@ -1195,79 +1217,95 @@ BEGIN_METHOD (PDFPAGE_find,GB_STRING Text; GB_BOOLEAN Sensitive;)
 
 	if (!MISSING(Sensitive)) sensitive=VARG(Sensitive);
 
+	#if POPPLER_VERSION_0_20
+	textdev = new TextOutputDev (NULL, true, 0, false, false);
+	THIS->page->display (textdev, 72, 72, 0, false, false, false);
+	#else
 	textdev = new TextOutputDev (NULL, true, false, false);
 	THIS->page->display (textdev, 72, 72, 0, false, false, false, THIS->doc->getCatalog());
+	#endif
 
 	if (THIS->Found) { GB.FreeArray(POINTER(&THIS->Found)); THIS->Found=NULL; }
 
-	while (textdev->findText (block,nlen,gFalse,gTrue,gTrue,gFalse,sensitive,gFalse,&x0,&y0,&x1,&y1))
+	count = 0;
+	#if POPPLER_VERSION_0_20
+	while (textdev->findText (block,nlen,false,true,true,false,sensitive,false,false,&x0,&y0,&x1,&y1))
+	#else
+	while (textdev->findText (block,nlen,false,true,true,false,sensitive,false,&x0,&y0,&x1,&y1))
+	#endif
 	{
-		if (!THIS->Found) {
-			GB.NewArray(POINTER(&THIS->Found),sizeof(FoundRect),1);
-		}		
-		else {
+		if (!THIS->Found)
+			GB.NewArray(POINTER(&THIS->Found),sizeof(CPDFFIND),1);
+		else
 			GB.Add(POINTER(&THIS->Found));
-		}
 
-		el=(FoundRect*)&((FoundRect*)THIS->Found)[GB.Count(POINTER(THIS->Found))-1];
+		el = &(THIS->Found[count++]); //(CPDFFIND*)&((CPDFFIND*)THIS->Found)[GB.Count(POINTER(THIS->Found))-1];
 		
-		switch (THIS->rotation)
+		x = x0;
+		y = y0;
+		w = x1 - x0;
+		h = y1 - y0;
+
+		wp = THIS->page->getMediaWidth();
+		hp = THIS->page->getMediaHeight();
+		rotation = THIS->page->getRotate();
+		if (rotation == 90 || rotation == 270)
 		{
-			case 0:
-				el->x0=(int32_t)(x0*THIS->scale);
-				el->y0=(int32_t)(y0*THIS->scale);
-				el->x1=(int32_t)((x1-x0)*THIS->scale);
-				el->y1=(int32_t)((y1-y0)*THIS->scale);
-				break;
-
-			case 90:
-				el->y1=(int32_t)((x1-x0)*THIS->scale);
-				el->x1=(int32_t)(y1-y0);
-				el->y0=(int32_t)(x0*THIS->scale);
-				el->x0=(int32_t)((THIS->page->getMediaHeight()-y0-el->x1)*THIS->scale);
-				el->x1=(int32_t)(el->x1*THIS->scale);
-				break;
-
-			case 180:
-				el->x1=(int32_t)(x1-x0);
-				el->y1=(int32_t)(y1-y0);
-				el->x0=(int32_t)((THIS->page->getMediaWidth()-x0-el->x1)*THIS->scale);
-				el->y0=(int32_t)((THIS->page->getMediaHeight()-y0-el->y1)*THIS->scale);
-				el->x1=(int32_t)(el->x1*THIS->scale);
-				el->y1=(int32_t)(el->y1*THIS->scale);
-				break;
-
-			case 270:
-				el->x1=(int32_t)((y1-y0)*THIS->scale);
-				el->y1=(int32_t)(x1-x0);
-				el->x0=(int32_t)(y0*THIS->scale);
-				el->y0=(int32_t)((THIS->page->getMediaWidth()-x0-el->y1)*THIS->scale);
-				el->y1=(int32_t)(el->y1*THIS->scale);
-				break;
-
+			x0 = wp; wp = hp; hp = x0;
 		}
+
+		rotation = THIS->rotation; //get_rotation(THIS);
+		while (rotation > 0)
+		{
+			x0 = wp; wp = hp; hp = x0;
+
+			x0 = wp - y - h;
+			y0 = x;
+
+			x = w; w = h; h = x;
+
+			x = x0;
+			y = y0;
+
+			rotation -= 90;
+		}
+
+		el->x0 = x * THIS->scale;
+		el->y0 = y * THIS->scale;
+		el->x1 = w * THIS->scale;
+		el->y1 = h * THIS->scale;
 	}
 
 	delete textdev;
 
+	GB.ReturnBoolean(count == 0);
 
 END_METHOD
 
 
-BEGIN_METHOD (PDFPAGERESULT_get,GB_INTEGER Index;)
+BEGIN_METHOD(PDFPAGERESULT_get,GB_INTEGER Index)
 
-	bool bok=true;
+	CPDFRECT *rect;
+	CPDFFIND *el;
+	int index;
 
-	if (!THIS->Found) bok=false;
-	else
+	index = VARG(Index);
+	
+	if (!THIS->Found || index < 0 || index >= GB.Count(THIS->Found))
 	{
-		if (VARG(Index)<0) bok=false;
-		if (VARG(Index)>= GB.Count(POINTER(THIS->Found)) ) bok=false;
+		GB.Error("Out of bounds");
+		return;
 	}
-	if (!bok) { GB.Error("Out of bounds"); GB.ReturnNull(); return; }
 
-	THIS->fcurrent=VARG(Index);
-	RETURN_SELF();
+	el = &(THIS->Found[index]);
+	rect = create_rect();
+	
+	rect->x = el->x0;
+	rect->y = el->y0;
+	rect->w = el->x1;
+	rect->h = el->y1;
+	
+	GB.ReturnObject(rect);
 
 END_METHOD
 
@@ -1278,33 +1316,6 @@ BEGIN_PROPERTY (PDFPAGERESULT_count)
 
 END_PROPERTY
 
-BEGIN_PROPERTY (PDFPAGERESULT_width)
-
-	FoundRect *el=(FoundRect*)&((FoundRect*)THIS->Found)[THIS->fcurrent];
-	GB.ReturnInteger((int32_t)el->x1);
-
-END_PROPERTY
-
-BEGIN_PROPERTY (PDFPAGERESULT_height)
-
-	FoundRect *el=(FoundRect*)&((FoundRect*)THIS->Found)[THIS->fcurrent];
-	GB.ReturnInteger((int32_t)el->y1);
-
-END_PROPERTY
-
-BEGIN_PROPERTY (PDFPAGERESULT_left)
-
-	FoundRect *el=(FoundRect*)&((FoundRect*)THIS->Found)[THIS->fcurrent];
-	GB.ReturnInteger((int32_t)el->x0);
-
-END_PROPERTY
-
-BEGIN_PROPERTY (PDFPAGERESULT_top)
-
-	FoundRect *el=(FoundRect*)&((FoundRect*)THIS->Found)[THIS->fcurrent];
-	GB.ReturnInteger((int32_t)el->y0);
-
-END_PROPERTY
 
 /**********************************************************************
 
@@ -1312,24 +1323,30 @@ Gambas Interface
 
 ***********************************************************************/
 
-
-GB_DESC PdfResultItemDesc[]=
+GB_DESC PdfRectDesc[] =
 {
-	GB_DECLARE(".PdfResultItem",0), GB_VIRTUAL_CLASS(),
-
-	GB_PROPERTY_READ("Left","i",PDFPAGERESULT_left),
-	GB_PROPERTY_READ("Top","i",PDFPAGERESULT_top),
-	GB_PROPERTY_READ("Width","i",PDFPAGERESULT_width),
-	GB_PROPERTY_READ("Height","i",PDFPAGERESULT_height),
-
+	GB_DECLARE("PdfRect", sizeof(CPDFRECT)), GB_NOT_CREATABLE(),
+	
+	GB_PROPERTY_READ("X", "f", PdfRect_X),
+	GB_PROPERTY_READ("Y", "f", PdfRect_Y),
+	GB_PROPERTY_READ("Width", "f", PdfRect_Width),
+	GB_PROPERTY_READ("Height", "f", PdfRect_Height),
+	GB_PROPERTY_READ("W", "f", PdfRect_Width),
+	GB_PROPERTY_READ("H", "f", PdfRect_Height),
+	GB_PROPERTY_READ("Left", "f", PdfRect_X),
+	GB_PROPERTY_READ("Top", "f", PdfRect_Y),
+	GB_PROPERTY_READ("Right", "f", PdfRect_Right),
+	GB_PROPERTY_READ("Bottom", "f", PdfRect_Bottom),
+	
 	GB_END_DECLARE
 };
 
+
 GB_DESC PdfResultDesc[]=
 {
-	GB_DECLARE(".PdfResult",0), GB_VIRTUAL_CLASS(),
+	GB_DECLARE(".PdfDocumentPage.Result",0), GB_VIRTUAL_CLASS(),
 
-	GB_METHOD("_get",".PdfResultItem",PDFPAGERESULT_get,"(Index)i"),
+	GB_METHOD("_get","PdfRect",PDFPAGERESULT_get,"(Index)i"),
 	GB_PROPERTY_READ("Count","i",PDFPAGERESULT_count),
 
 	GB_END_DECLARE
@@ -1338,44 +1355,41 @@ GB_DESC PdfResultDesc[]=
 
 GB_DESC PdfLinkDataDesc[]=
 {
-	GB_DECLARE(".PdfLinkData",0), GB_VIRTUAL_CLASS(),
+	GB_DECLARE(".PdfDocumentPage.Link.Data",0), GB_VIRTUAL_CLASS(),
 
 	GB_PROPERTY_READ("Type","i",PDFPAGELINKDATA_type),
 	GB_PROPERTY_READ("Target","s",PDFPAGELINKDATA_uri),
 	GB_PROPERTY_READ("Parameters","s",PDFPAGELINKDATA_parameters),
 	GB_PROPERTY_READ("Page","i",PDFPAGELINKDATA_page),
-	GB_PROPERTY_READ("Left","i",PDFPAGELINKDATA_left),
-	GB_PROPERTY_READ("Top","i",PDFPAGELINKDATA_top),
-	GB_PROPERTY_READ("Right","i",PDFPAGELINKDATA_right),
-	GB_PROPERTY_READ("Bottom","i",PDFPAGELINKDATA_bottom),
 	GB_PROPERTY_READ("Zoom","f",PDFPAGELINKDATA_zoom),
+	GB_PROPERTY_READ("Rect", "PdfRect", PdfPageLinkData_Rect),
 
 	GB_END_DECLARE
 };
 
+
 GB_DESC PdfLinkDesc[]=
 {
-	GB_DECLARE(".PdfLink",0), GB_VIRTUAL_CLASS(),
+	GB_DECLARE(".PdfDocumentPage.Link",0), GB_VIRTUAL_CLASS(),
 
-	GB_PROPERTY_READ("Left","i",PDFPAGELINK_left),
-	GB_PROPERTY_READ("Top","i",PDFPAGELINK_top),
-	GB_PROPERTY_READ("Width","i",PDFPAGELINK_width),
-	GB_PROPERTY_READ("Height","i",PDFPAGELINK_height),
-	GB_PROPERTY_READ("Data", ".PdfLinkData", PDFPAGELINKDATA_check),
+	GB_PROPERTY_READ("Rect", "PdfRect", PdfPageLink_rect),
+	GB_PROPERTY_READ("Data",".PdfDocumentPage.Link.Data", PDFPAGELINKDATA_check),
 
 	GB_END_DECLARE
 };
 
+
 GB_DESC PdfIndexDesc[]=
 {
-	GB_DECLARE(".PdfIndex",0), GB_VIRTUAL_CLASS(),
+	GB_DECLARE(".PdfDocument.Index",0), GB_VIRTUAL_CLASS(),
 
 	GB_PROPERTY("Expanded","b",PDFINDEX_is_open),
 	GB_PROPERTY_READ("Count","i",PDFINDEX_count),
 	GB_PROPERTY_READ("HasChildren","b",PDFINDEX_has_children),
 	GB_PROPERTY_READ("Title","s",PDFINDEX_title),
+	GB_PROPERTY_READ("Text","s",PDFINDEX_title),
 
-	GB_PROPERTY_READ("Data", ".PdfLinkData", PDFPAGELINKDATA_check),
+	GB_PROPERTY_READ("Data", ".PdfDocumentPage.Link.Data", PDFPAGELINKDATA_check),
 	GB_METHOD("MovePrevious","b",PDFINDEX_prev,0),
 	GB_METHOD("MoveNext","b",PDFINDEX_next,0),
 	GB_METHOD("MoveChild","b",PDFINDEX_child,0),
@@ -1388,30 +1402,30 @@ GB_DESC PdfIndexDesc[]=
 
 GB_DESC PdfPageDesc[]=
 {
-	GB_DECLARE(".PdfPage",0), GB_VIRTUAL_CLASS(),
+	GB_DECLARE(".PdfDocumentPage",0), GB_VIRTUAL_CLASS(),
 
 	GB_PROPERTY_READ("W","f",PDFPAGE_width),
 	GB_PROPERTY_READ("H","f",PDFPAGE_height),
 	GB_PROPERTY_READ("Width","f",PDFPAGE_width),
 	GB_PROPERTY_READ("Height","f",PDFPAGE_height),
-	GB_PROPERTY_READ("Picture","Picture",PDFPAGE_property_picture),
+	
 	GB_PROPERTY_READ("Image","Image",PDFPAGE_property_image),
-	GB_PROPERTY_SELF("Result",".PdfResult"),
+	GB_PROPERTY_SELF("Result",".PdfDocumentPage.Result"),
 
 	GB_METHOD("GetImage","Image",PDFPAGE_image,"[(X)i(Y)i(Width)i(Height)i]"),
-	GB_METHOD("GetPicture","Picture",PDFPAGE_picture,"[(X)i(Y)i(Width)i(Height)i]"),
 	GB_METHOD("Find","b",PDFPAGE_find,"(Text)s[(CaseSensitive)b]"),
 	GB_METHOD("Select","s",PDFPAGE_select,"[(X)i(Y)i(W)i(H)i]"),
 
-	GB_METHOD("_get",".PdfLink",PDFPAGELINKS_get,"(Index)i"),
+	GB_METHOD("_get",".PdfDocumentPage.Link",PDFPAGELINKS_get,"(Index)i"),
 	GB_PROPERTY_READ("Count","i",PDFPAGELINKS_count),
 
 	GB_END_DECLARE
 };
 
+
 GB_DESC PdfDocumentInfo[] =
 {
-	GB_DECLARE(".PdfInfo",0), GB_VIRTUAL_CLASS(),
+	GB_DECLARE(".PdfDocument.Info",0), GB_VIRTUAL_CLASS(),
 
 	GB_PROPERTY_READ("Title","s",PDFINFO_title),
 	GB_PROPERTY_READ("Format","s",PDFINFO_format),
@@ -1433,9 +1447,9 @@ GB_DESC PdfDocumentInfo[] =
 	GB_END_DECLARE
 };
 
+
 GB_DESC PdfLayoutDesc[] =
 {
-
   GB_DECLARE("PdfLayout", 0), GB_NOT_CREATABLE(),
 
   GB_CONSTANT("Unset","i",Catalog::pageLayoutNone),
@@ -1466,7 +1480,6 @@ GB_DESC PdfModeDesc[] =
 
 GB_DESC PdfDocumentDesc[] =
 {
-
   GB_DECLARE("PdfDocument", sizeof(CPDFDOCUMENT)),
 
   GB_CONSTANT("Unknown","i",actionUnknown),  /* unknown action */
@@ -1487,7 +1500,7 @@ GB_DESC PdfDocumentDesc[] =
 
   GB_METHOD("Open",0,PDFDOCUMENT_open,"(File)s"),
   GB_METHOD("Close",0,PDFDOCUMENT_close,0),
-  GB_METHOD("_get",".PdfPage",PDFDOCUMENT_get,"(Index)i"),
+  GB_METHOD("_get",".PdfDocumentPage",PDFDOCUMENT_get,"(Index)i"),
 
   GB_PROPERTY("Zoom", "f", PDFDOCUMENT_scale),
   GB_PROPERTY("Orientation", "i", PDFDOCUMENT_rotation),
@@ -1495,8 +1508,8 @@ GB_DESC PdfDocumentDesc[] =
   GB_PROPERTY_READ("Ready","b",PDFDOCUMENT_ready),
   GB_PROPERTY_READ("Count","i",PDFDOCUMENT_count),
   GB_PROPERTY_READ("HasIndex","b",PDFDOCUMENT_has_index),
-  GB_PROPERTY_READ("Index",".PdfIndex",PDFDOCUMENT_index),
-  GB_PROPERTY_READ("Info",".PdfInfo",PDFDOCUMENT_info),
+  GB_PROPERTY_READ("Index",".PdfDocument.Index",PDFDOCUMENT_index),
+  GB_PROPERTY_READ("Info",".PdfDocument.Info",PDFDOCUMENT_info),
 
   GB_END_DECLARE
 };
