@@ -2,14 +2,11 @@
 
   SDLgfx.cpp
 
-  Gambas extension using SDL
-
   (c) 2006 Laurent Carlier <lordheavy@users.sourceforge.net>
-           Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -19,27 +16,27 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
 #include "SDLgfx.h"
+#include "SDLerror.h"
 #include "SDLcore.h"
 #include "SDLapp.h"
+#include "SDLwindow.h"
+#include "SDLsurface.h"
+#include "SDLtexture.h"
 
 #include <iostream>
 #include <math.h>
 
-// texture status values
-#define TEXTURE_OK		(0)
-#define TEXTURE_TO_RELOAD	(1<<0)
-#define TEX_TO_REDEFINE 	(1<<1)
-
 // for ellipses
 #define PI 3.14159265359
 
 // debug
-// #define DEBUGGFX
+// #define DEBUG_GFX
 
 // fill patterns
 static GLubyte VertPattern[] = {0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 
@@ -271,111 +268,35 @@ static void SetFillPattern(int value)
 
 SDLgfx::SDLgfx(SDLwindow *window)
 {
-	hSurfaceInfo = window->hSurfaceInfo;
-	hWindowDraw = true;
-	hContextDefined = false;
+	hTex = NULL;
 	resetGfx();
 }
 
 SDLgfx::SDLgfx(SDLsurface *surface)
 {
-	hSurfaceInfo = surface->hSurfaceInfo;
-	hWindowDraw = false;
-	hContextDefined = false;
-	resetGfx();
-
-	/* Does the surface already got a context (a Pbuffer is already defined ?) */
-	if (hContext)
-		return;
-
-	/* we define the pixel buffer if needed */
-	int scrnum;
-	GLXFBConfig *fbconfig;
-	XVisualInfo *visinfo;
-	int nitems;
-	Display *disp = SDLapp->X11appDisplay();
-
-	int attrib[] = 
-	{
-		GLX_DOUBLEBUFFER,  False,
-		GLX_RED_SIZE,      1,
-		GLX_GREEN_SIZE,    1,
-		GLX_BLUE_SIZE,     1,
-		GLX_RENDER_TYPE,   GLX_RGBA_BIT,
-		GLX_DRAWABLE_TYPE, GLX_PBUFFER_BIT | GLX_WINDOW_BIT,
-		None
-	};
-
-	int pbufAttrib[] = 
-	{
-		GLX_PBUFFER_WIDTH,   hSurface->w,
-		GLX_PBUFFER_HEIGHT,  hSurface->h,
-		GLX_LARGEST_PBUFFER, False,
-		None
-	};
-
-	SDLapp->LockX11();
-
-	scrnum = DefaultScreen(disp);
-	fbconfig = glXChooseFBConfig(disp, scrnum, attrib, &nitems);
-
-	if (!fbconfig)
-	{
-        	SDLcore::RaiseError("SDLgfx: error, couldn't get fbconfig");
-        	return;
-	}
-
-	hPbuffer = glXCreatePbuffer(disp, fbconfig[0], pbufAttrib);
-	visinfo = glXGetVisualFromFBConfig(disp, fbconfig[0] );
-
-	if (!visinfo )
-	{
-        	SDLcore::RaiseError("SDLgfx: error, couldn't get an RGBA, double-buffered visual");
-		return;
-	}
-
 	if (!SDLcore::GetWindow())
 	{
-        	SDLcore::RaiseError("SDLgfx: window not defined, will be fixed later !");
+		SDLerror::RaiseError("Window need to be opened first !");
 		return;
 	}
 
-	hContext = glXCreateContext(disp, visinfo, SDLcore::GetWindow()->hSurfaceInfo->Ctx, GL_TRUE);
-
-	if (!hContext )
-	{
-		SDLcore::RaiseError("SDLgfx: error, Call to glXCreateContext failed!");
-		return;
-	}
-
-	XFree(fbconfig);
-	XFree(visinfo);
-	hContextDefined = true;
-
-	SDLapp->UnlockX11();
-}
-
-SDLgfx::~SDLgfx()
-{
-	if (hWindowDraw)
-		return;
-
-	if (!hContextDefined)
-		return;
-
-	Display *disp = SDLapp->X11appDisplay();
-	glXDestroyContext(disp, hContext);
-	glXDestroyPbuffer(disp, hPbuffer);
+	hTex = surface->GetTexture();
+	resetGfx();
 }
 
 void SDLgfx::resetGfx(void)
 {
-	hForeColor = 0xFFFFFFFF;
-	hBackColor = 0x00000000;
 	hLine = SDL::SolidLine;
 	hLineWidth = 1;
 	hFill = SDL::NoFill;
-	hTextureStatus = TEXTURE_TO_RELOAD;
+	rotx = roty = rotz = 0;
+	scalex = scaley = 1.0f;
+}
+
+void SDLgfx::SetColor(Uint32 color)
+{
+	glColor4f((GLfloat((color >> 16) & 0xFF)/255), (GLfloat((color >> 8) & 0xFF)/255), (GLfloat(color  & 0xFF)/255),
+                (GLfloat(~(color >> 24) & 0xFF)/255));
 }
 
 void SDLgfx::SetLineStyle(int style)
@@ -396,131 +317,67 @@ void SDLgfx::SetFillStyle(int style)
 
 void SDLgfx::Clear(void)
 {
-	if (!hSurface)
-		return;
+	SetContext();
 
-	Uint32 myColor = hBackColor;
-
-	if (hSurface->flags & SDL_OPENGL)
-	{
-		glClearColor((GLfloat((hBackColor >> 24) & 0xFF)/255), (GLfloat((hBackColor >> 16) & 0xFF)/255), (GLfloat((hBackColor >> 8) & 0xFF)/255), 1.0f);
-		glClear(GL_COLOR_BUFFER_BIT);
-	}
-	else
-	{
-		Uint32 FillColor = SDL_MapRGB(hSurface->format, ((hBackColor >> 24) & 0xFF), ((hBackColor >> 16) & 0xFF), ((hBackColor >> 8) & 0xFF));
-
-		if (SDL_FillRect(hSurface, NULL, FillColor)<0)
-			SDLcore::RaiseError(SDL_GetError());
-	}
-
-	hBackColor = myColor;
-	hTextureStatus = TEXTURE_TO_RELOAD;
+ 	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
+	glClear(GL_COLOR_BUFFER_BIT);
 }
 
 void SDLgfx::DrawPixel(int x, int y)
 {
-	if (!hSurface)
-		return;
+	SetContext();
+
+	glPushAttrib(GL_ENABLE_BIT);
+	glEnable(GL_BLEND);
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
 	glBegin(GL_POINTS);
-	glColor4f((GLfloat((hForeColor >> 24) & 0xFF)/ 255), (GLfloat((hForeColor >> 16) & 0xFF)/255), (GLfloat((hForeColor >> 8) & 0xFF)/255), (GLfloat(hForeColor  & 0xFF)/255));
 	glVertex2i(x, y);
 	glEnd();
 
-	hTextureStatus = TEXTURE_TO_RELOAD;
+	glPopAttrib();
 }
 
 void SDLgfx::DrawLine(int x1, int y1, int x2, int y2)
 {
-	if (!hSurface)
-		return;
-
 	if (!hLine) // SDLgfx::NoLine
 		return;
 
-	if (hSurface->flags & SDL_OPENGL)
-	{
-		glPushAttrib(GL_ALL_ATTRIB_BITS);
+	SetContext();
 
-		SetLinePattern(hLine);
-		glLineWidth(GLfloat(hLineWidth));
+	glPushAttrib(GL_ENABLE_BIT);
+	glEnable(GL_BLEND);
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
-		glBegin(GL_LINES);
-		glColor4f((GLfloat((hForeColor >> 24) & 0xFF)/ 255), (GLfloat((hForeColor >> 16) & 0xFF)/255), (GLfloat((hForeColor >> 8) & 0xFF)/255), (GLfloat(hForeColor  & 0xFF)/255));
-		glVertex2i(x1, y1);
-		glVertex2i(x2, y2);
-		glEnd();
+	SetLinePattern(hLine);
+	glLineWidth(GLfloat(hLineWidth));
 
-		glPopAttrib();
-	}
+	glBegin(GL_LINES);
+	glVertex2i(x1, y1);
+	glVertex2i(x2, y2);
+	glEnd();
 
-	hTextureStatus = TEXTURE_TO_RELOAD;
+	glPopAttrib();
 }
 
 void SDLgfx::DrawRect(int x, int y, int w, int h)
 {
-	if (!hSurface)
-		return;
-
 	if (!hFill && !hLine)
 		return;
 
-	if (hSurface->flags & SDL_OPENGL)
-	{
-		glPushAttrib(GL_ALL_ATTRIB_BITS);
+	SetContext();
 
-		SetFillPattern(hFill);
-
-		glColor4f((GLfloat((hForeColor >> 24) & 0xFF)/ 255), (GLfloat((hForeColor >> 16) & 0xFF)/255), (GLfloat((hForeColor >> 8) & 0xFF)/255), (GLfloat(hForeColor  & 0xFF)/255));
-		glBegin(GL_QUADS);
-		glVertex2i(x, y);
-		glVertex2i(x+w, y);
-		glVertex2i(x+w, y+h);
-		glVertex2i(x, y+h);
-		glEnd();
-
-		if (hFill>SDL::SolidFill)
-		{
-			SetFillPattern(SDL::NoFill);
-			SetLinePattern(hLine);
-			glLineWidth(GLfloat(hLineWidth));
-
-			glBegin(GL_QUADS);
-			glVertex2i(x, y);
-			glVertex2i(x+w, y);
-			glVertex2i(x+w, y+h);
-			glVertex2i(x, y+h);
-			glEnd();
-		}
-
-		glPopAttrib();
-	}
-}
-
-void SDLgfx::DrawEllipse(int x, int y, int w, int h)
-{
-	if (!hSurface)
-		return;
-
-	if (!hFill && !hLine)
-		return;
-
-	double angle;
-	double step = 2 * PI / 360;
-
-	glPushAttrib(GL_ALL_ATTRIB_BITS);
+	glPushAttrib(GL_ENABLE_BIT);
+	glEnable(GL_BLEND);
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
 	SetFillPattern(hFill);
 
-	glColor4f((GLfloat((hForeColor >> 24) & 0xFF)/ 255), (GLfloat((hForeColor >> 16) & 0xFF)/255), (GLfloat((hForeColor >> 8) & 0xFF)/255), (GLfloat(hForeColor  & 0xFF)/255));
-
-	glBegin(GL_POLYGON);
-
-	for (angle=0; angle < 2 * PI; angle += step)
-	{
-		glVertex2d(x + (w * cos(angle)), y + (h * sin(angle)));
-	}
+	glBegin(GL_QUADS);
+	glVertex2i(x, y);
+	glVertex2i(x+w, y);
+	glVertex2i(x+w, y+h);
+	glVertex2i(x, y+h);
 	glEnd();
 
 	if (hFill>SDL::SolidFill)
@@ -529,53 +386,95 @@ void SDLgfx::DrawEllipse(int x, int y, int w, int h)
 		SetLinePattern(hLine);
 		glLineWidth(GLfloat(hLineWidth));
 
-		glBegin(GL_POLYGON);
-		for (angle=0; angle < 2 * PI; angle += step)
-		{
-			glVertex2d(x + (w * cos(angle)), y + (h * sin(angle)));
-		}
+		glBegin(GL_QUADS);
+		glVertex2i(x, y);
+		glVertex2i(x+w, y);
+		glVertex2i(x+w, y+h);
+		glVertex2i(x, y+h);
 		glEnd();
 	}
 
+	SetFillPattern(SDL::SolidFill);
 	glPopAttrib();
 }
 
-void SDLgfx::Blit(SDLsurface *surface, int x, int y, int srcX, int srcY,
-		 int srcWidth, int srcHeight, int width, int height)
+void SDLgfx::DrawEllipse(int x, int y, int w, int h)
 {
-	if (!hSurface)
+	if (!hFill && !hLine)
 		return;
 
-	// will keep current SDL_INFO struct
-	SDL_INFO *tmpInfo = hSurfaceInfo;
-	int myWidth = 0, myHeight = 0;
+	SetContext();
 
-	// we will work with the SDLsurface SDL_INFO struct
-	hSurfaceInfo = surface->hSurfaceInfo;
+	double angle;
+	double step = 2 * PI / 360;
 
-	if (!hSurface)
-		goto _endblit;
+	glPushAttrib(GL_ENABLE_BIT);
+	glEnable(GL_BLEND);
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
-	if ((srcX > hSurface->w) || (srcY > hSurface->h))
-		goto _endblit;
+	glTranslatef(x, y, 0.0f);
+	SetFillPattern(hFill);
 
-	if ((srcHeight<0) || ((srcY + srcHeight) > hSurface->h))
-		myHeight = hSurface->h - srcY;
+	glBegin(GL_POLYGON);
+	for (angle=0; angle < 2 * PI; angle += step)
+		glVertex2d(w * cos(angle), h * sin(angle));
+	glEnd();
+
+	if (hFill>SDL::SolidFill)
+	{
+		SetFillPattern(SDL::NoFill);
+		SetLinePattern(hLine);
+		glLineWidth(GLfloat(hLineWidth));
+		glBegin(GL_POLYGON);
+		for (angle=0; angle < 2 * PI; angle += step)
+			glVertex2d(w * cos(angle), h * sin(angle));
+		glEnd();
+	}
+
+	SetFillPattern(SDL::SolidFill);
+	glPopAttrib();
+	glLoadIdentity();
+}
+
+void SDLgfx::Blit(SDLsurface *surface, int x, int y, int srcX, int srcY,
+		 int srcWidth, int srcHeight, int width, int height, bool no_filter)
+{
+	if ((srcX > surface->GetWidth()) || (srcY > surface->GetHeight()))
+		return;
+
+	if (!surface->GetWidth() || !surface->GetHeight())
+		return;
+	
+	SDL_Surface *destsurf = GetDestSurface();
+
+	if ((x > destsurf->w) || (y > destsurf->h))
+		return;
+
+	SetContext();
+	texinfo info;
+
+	glPushAttrib(GL_ENABLE_BIT);
+	SDLtexture *texture = surface->GetTexture();
+	texture->GetAsTexture(&info);
+
+	GLfloat myWidth = 0, myHeight = 0;
+
+	if ((srcHeight<0) || ((srcY + srcHeight) > surface->GetHeight()))
+		myHeight = surface->GetHeight() - srcY;
 	else
 		myHeight = srcHeight;
 
-	if ((srcWidth<0) || ((srcX + srcWidth) > hSurface->w))
-		myWidth = hSurface->w - srcX;
+	if ((srcWidth<0) || ((srcX + srcWidth) > surface->GetWidth()))
+		myWidth = surface->GetWidth() - srcX;
 	else
 		myWidth = srcWidth;
 
 	GLdouble myTexX, myTexY, myTexHeight, myTexWidth;
-	ManageTexture();
 
-	myTexX = ((srcX * hTextureWidth) / hSurface->w);
-	myTexY = ((srcY * hTextureHeight) / hSurface->h);
-	myTexWidth = (((srcX + myWidth)* hTextureWidth) / hSurface->w);
-	myTexHeight = (((srcY + myHeight)* hTextureHeight) / hSurface->h);
+	myTexX = ((srcX * info.Width) / surface->GetWidth());
+	myTexY = ((srcY * info.Height) / surface->GetHeight());
+	myTexWidth = (((srcX + myWidth)* info.Width) / surface->GetWidth());
+	myTexHeight = (((srcY + myHeight)* info.Height) / surface->GetHeight());
 
 	if (width != -1)
 		myWidth = width;
@@ -583,105 +482,52 @@ void SDLgfx::Blit(SDLsurface *surface, int x, int y, int srcX, int srcY,
 	if (height != -1)
 		myHeight = height;
 
+	myWidth = myWidth / 2;
+	myHeight = myHeight / 2;
+	
+	glEnable(GL_BLEND);
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+	glEnable(GL_TEXTURE_2D);
+	glBindTexture(GL_TEXTURE_2D, info.Index);
+
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, no_filter ? GL_NEAREST : GL_LINEAR);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, no_filter ? GL_NEAREST : GL_LINEAR);
+	
+	glTranslatef(myWidth + x, myHeight + y, 0.0f);
+	glRotatef(rotz, 0, 0, 1);
+	glScalef(scalex, scaley, 0.0f);
+	
 	glBegin(GL_QUADS);
-	glColor4f(1.0, 1.0, 1.0, 1.0);
 	glTexCoord2d(myTexX, myTexY);
-	glVertex2i(x, y);
+	glVertex2f(-myWidth, -myHeight);
 
 	glTexCoord2d(myTexX, myTexHeight);
-	glVertex2i(x, y + myHeight);
+	glVertex2f(-myWidth, myHeight);
 
 	glTexCoord2d(myTexWidth, myTexHeight);
-	glVertex2i(x + myWidth, y + myHeight);
+	glVertex2f(myWidth, myHeight);
 
 	glTexCoord2d(myTexWidth, myTexY);
-	glVertex2i(x + myWidth, y);
+	glVertex2f(myWidth, -myHeight);
 	glEnd();
 
-_endblit:
-	hSurfaceInfo = tmpInfo;
+	glPopAttrib();
+	glLoadIdentity();
 }
 
-static int power_of_two(int input)
+void SDLgfx::SetContext()
 {
-	int value = 1;
-
-	while ( value < input )
-	{
-		value <<= 1;
-	}
-	return value;
+	if (!hTex)
+		SDLcore::GetWindow()->Select();
+	else
+		hTex->Select();
 }
 
-void SDLgfx::ManageTexture()
+SDL_Surface *SDLgfx::GetDestSurface()
 {
-	if (!hTexture)
-	{
-		glGenTextures(1, &hTexture);
-		hTextureStatus = TEXTURE_TO_RELOAD;
-	}
+	if (!hTex)
+		return SDLcore::GetWindow()->GetSdlSurface();
+	else
+		return hTex->GetSurface()->GetSdlSurface();
+}
 
-	glEnable(GL_TEXTURE_2D);
-	glBindTexture(GL_TEXTURE_2D, hTexture);
-
-	if (hTextureStatus & TEXTURE_TO_RELOAD)
-	{
-		#ifdef DEBUGGFX
-		std::cout << "Loading texture " << hTextureIndex << std::endl;
-		#endif
-
-		int w, h;
-		SDL_Surface *image;
-		Uint32 saved_flags;
-		Uint8  saved_alpha;
-
-		/* Use the surface width and height expanded to powers of 2 */
-		w = power_of_two(hSurface->w);
-		h = power_of_two(hSurface->h);
-		hTextureWidth = GLdouble(hSurface->w) / w;  /* Max X */
-		hTextureHeight = GLdouble(hSurface->h) / h;  /* Max Y */
-
-		image = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32,
-#if SDL_BYTEORDER == SDL_LIL_ENDIAN /* OpenGL RGBA masks */
-			0x000000FF, 
-			0x0000FF00, 
-			0x00FF0000, 
-			0xFF000000
-#else
-			0xFF000000,
-			0x00FF0000, 
-			0x0000FF00, 
-			0x000000FF
-#endif
-			);
-
-		if ( image == NULL )
-			return;
-
-		/* Save the alpha blending attributes */
-		saved_flags = hSurface->flags & (SDL_SRCALPHA | SDL_RLEACCELOK);
-		saved_alpha = hSurface->format->alpha;
-
-		if ((saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA)
-			SDL_SetAlpha(hSurface, 0, 0);
-
-		/* Copy the surface into the GL texture image */
-		SDL_BlitSurface(hSurface, NULL, image, NULL);
-
-		/* Restore the alpha blending attributes */
-		if ((saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA )
-			SDL_SetAlpha(hSurface, saved_flags, saved_alpha);
-
-		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, 
-			GL_UNSIGNED_BYTE, image->pixels);
-		/* Use of CLAMP_TO_EDGE, without it give a black line around the texture
-		with DRI radeon drivers */
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-		SDL_FreeSurface(image); /* No longer needed */
-
-		hTextureStatus = TEXTURE_OK;
-	}
-}	
