@@ -1,14 +1,12 @@
 /***************************************************************************
 
-	archive.c
+	gbc_archive.c
 
-	The GAMBAS Archiver
-
-	(c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 1, or (at your option)
+	the Free Software Foundation; either version 2, or (at your option)
 	any later version.
 
 	This program is distributed in the hope that it will be useful,
@@ -18,7 +16,8 @@
 
 	You should have received a copy of the GNU General Public License
 	along with this program; if not, write to the Free Software
-	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+	MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -34,7 +33,6 @@
 
 #include <sys/stat.h>
 #include <sys/types.h>
-
 #include <unistd.h>
 
 #include "gb_common.h"
@@ -48,15 +46,19 @@
 
 /*#define DEBUG*/
 
-PUBLIC char *ARCH_project;
-PUBLIC char *ARCH_project_name;
-PUBLIC char *ARCH_output = NULL;
-PUBLIC bool ARCH_verbose = FALSE;
-PUBLIC bool ARCH_swap = FALSE;
+#define TEMP_EXEC ".temp.gambas"
+
+char *ARCH_project;
+char *ARCH_project_name;
+char *ARCH_output = NULL;
+bool ARCH_verbose = FALSE;
+bool ARCH_swap = FALSE;
 
 static int arch_dir_pos;
 static TABLE *arch_table;
 static FILE *arch_file = NULL;
+
+#define ARCH_BUFFER_SIZE 4096
 static char *arch_buffer;
 
 static int pos_start;
@@ -79,9 +81,12 @@ static void write_short(ushort val)
 }
 
 
-static int get_pos(void)
+static long get_pos(void)
 {
-	return ftell(arch_file);
+	long pos = ftell(arch_file);
+	if (pos < 0)
+		THROW("Unable to get file position");
+	return (int)pos; // No archive file greater then 2 Go!
 }
 
 
@@ -104,39 +109,75 @@ static void write_string(const char *str, int len)
 
 static void make_executable(void)
 {
+	const char *err;
 	struct stat info;
 
-	if (stat(ARCH_output, &info) == 0)
-		if (chmod(ARCH_output, info.st_mode | S_IXUSR | S_IXGRP | S_IXOTH) == 0)
-		{
-			FILE_set_owner(ARCH_output, FILE_cat(FILE_get_dir(ARCH_project), ".project", NULL));
-			return;
-		}
+	FILE_chdir(FILE_get_dir(ARCH_output));
+	
+	// If we cannot make the archive executable, just print a warning. gbs creates an executable cache 
+	// inside /tmp, and /tmp may be mounted with the "noexec" flag.
+	
+	if (stat(TEMP_EXEC, &info) || chmod(TEMP_EXEC, info.st_mode | S_IXUSR | S_IXGRP | S_IXOTH))
+		fprintf(stderr, "gba: warning: cannot change executable permissions\n");
 
-	THROW("Cannot make executable: &1", strerror(errno));
+	FILE_set_owner(TEMP_EXEC, FILE_cat(FILE_get_dir(ARCH_project), ".project", NULL));
+
+	if (FILE_exist(ARCH_output) && unlink(ARCH_output))
+	{
+		err = "Cannot remove previous executable";
+		goto __ERROR;
+	}
+	
+	if (rename(TEMP_EXEC, ARCH_output))
+	{
+		err = "Cannot create executable";
+		goto __ERROR;
+	}
+	
+	return;
+
+__ERROR:
+
+	THROW("Cannot make executable: &1: &2", err, strerror(errno));
 }
 
 
-PUBLIC void ARCH_define_output(const char *path)
+void ARCH_define_output(const char *path)
 {
 	STR_free(ARCH_output);
+	
+	if (path && *path != '/')
+		path = FILE_cat(FILE_get_current_dir(), path, NULL);
+	else if (!path)
+		path = "";
+	
 	ARCH_output = STR_copy(path);
 }
 
-PUBLIC void ARCH_define_project(const char *project)
+static void print_separation()
+{
+	printf("---------------------------------------- ------------------------------ -------- -----\n");
+}
+
+void ARCH_define_project(const char *project)
 {
 	char *name;
 	char *dir;
+	const char *path;
 
 	if (project == NULL)
 		project = FILE_get_current_dir();
 
-	chdir(project);
+	FILE_chdir(project);
 	dir = STR_copy(FILE_get_current_dir());
 
 	arch_dir_pos = strlen(dir) + 1;
 
-	ARCH_project = STR_copy(FILE_cat(dir, ".project", NULL));
+	path = FILE_cat(dir, ".startup", NULL);
+	if (FILE_exist(path))
+		ARCH_project = STR_copy(path);
+	else
+		ARCH_project = STR_copy(FILE_cat(dir, ".project", NULL));
 
 	name = STR_copy(FILE_get_name(dir));
 
@@ -151,13 +192,13 @@ PUBLIC void ARCH_define_project(const char *project)
 }
 
 
-PUBLIC void ARCH_init(void)
+void ARCH_init(void)
 {
 	TABLE_create(&arch_table, sizeof(ARCH_SYMBOL), TF_NORMAL);
 
-	ALLOC(&arch_buffer, 4096, "ARCH_init");
+	ALLOC(&arch_buffer, ARCH_BUFFER_SIZE);
 
-	arch_file = fopen(ARCH_output, "w");
+	arch_file = fopen(FILE_cat(FILE_get_dir(ARCH_output), TEMP_EXEC, NULL), "w");
 	if (arch_file == NULL)
 		THROW("Cannot create temporary archive file: &1", ARCH_output);
 
@@ -171,7 +212,11 @@ PUBLIC void ARCH_init(void)
 	write_int(ARCH_VERSION);
 	
 	if (ARCH_verbose)
-		printf("Format version: %d\n", ARCH_VERSION);
+	{
+		print_separation();
+		printf("%-40.40s %-30.30s %8s %5s\n", "Path", "Abbrev. path","Size","Index");
+		print_separation();
+	}
 
 	pos_start = get_pos();
 	write_int(0);
@@ -191,15 +236,13 @@ static void compress_file_name(const char *src, int lsrc, char **dst, int *ldst)
 	char tpath2[PATH_MAX];
 	int len;
 	int ind;
-	ARCH_SYMBOL *sym;
 
-	
 	strncpy(tpath, src, lsrc);
 	tpath[lsrc] = 0;
 	len = lsrc;
 	
 	if (ARCH_verbose)
-		printf("%s", tpath); 	
+		printf("%-40.40s", tpath); 	
 	
 	for(;;)
 	{
@@ -207,25 +250,25 @@ static void compress_file_name(const char *src, int lsrc, char **dst, int *ldst)
 		if (!p)
 			break;
 			
-		if (!TABLE_find_symbol(arch_table, tpath, p - tpath, (SYMBOL **)(void *)&sym, &ind))
+		if (!TABLE_find_symbol(arch_table, tpath, p - tpath, &ind))
 		{
 			*p = 0;
-			THROW("&1: not in archive!\n", tpath);
+			THROW("&1: not in archive", tpath);
 		}
 	
-		len = sprintf(tpath2, "/%d:%s", ind, p + 1);
+		len = snprintf(tpath2, sizeof(tpath2), "/%d:%s", ind, p + 1);
 		strcpy(tpath, tpath2);
 	}
 
 	if (ARCH_verbose)
-		printf(" -> %s\n", tpath); 	
+		printf(" %-30.30s", tpath); 	
 		
 	*dst = tpath;
 	*ldst = len;
 }
 #endif
 
-PUBLIC void ARCH_exit(void)
+void ARCH_exit(void)
 {
 	int i;
 	ARCH_SYMBOL *sym;
@@ -240,7 +283,7 @@ PUBLIC void ARCH_exit(void)
 	for (i = 0; i < TABLE_count(arch_table); i++)
 	{
 		sym = (ARCH_SYMBOL *)TABLE_get_symbol(arch_table, i);
-		write_string(sym->symbol.name, sym->symbol.len);
+		write_string(sym->sym.name, sym->sym.len);
 	}
 
 	/* Write file names */
@@ -253,21 +296,29 @@ PUBLIC void ARCH_exit(void)
 	{
 		sym = (ARCH_SYMBOL *)TABLE_get_symbol(arch_table, i);
 		//write_short((ushort)i);
-		write_short(sym->symbol.sort);
-		write_short(sym->symbol.len);
 		write_int(pos_str);
+		write_int(sym->sym.len);
 		write_int(sym->pos);
 		write_int(sym->len);
 
-		pos_str += sym->symbol.len;
+		pos_str += sym->sym.len;
 	}
 
+	for (i = 0; i < TABLE_count(arch_table); i++)
+		write_short(arch_table->sort[i]);
+	
 	/* Close file */
 
 	fclose(arch_file);
 
 	make_executable();
 
+	if (ARCH_verbose)
+	{
+		print_separation();
+		printf("Writing archive format version %d:\n%s\n", ARCH_VERSION,ARCH_output);
+	}
+	
 	/* Free everything */
 
 	for (i = 0; i < TABLE_count(arch_table); i++)
@@ -279,11 +330,11 @@ PUBLIC void ARCH_exit(void)
 	STR_free(ARCH_project);
 	STR_free(ARCH_project_name);
 	
-	FREE(&arch_buffer, "ARCH_exit");
+	FREE(&arch_buffer);
 }
 
 
-PUBLIC int ARCH_add_file(const char *path)
+int ARCH_add_file(const char *path)
 {
 	char *rel_path;
 	ARCH_SYMBOL *sym;
@@ -301,12 +352,13 @@ PUBLIC int ARCH_add_file(const char *path)
 	len = strlen(rel_path);
 	#endif
 	
-	TABLE_add_symbol(arch_table, rel_path, len, (SYMBOL **)(void *)&sym, &ind);
+	TABLE_add_symbol(arch_table, rel_path, len, &ind);
+	sym = (ARCH_SYMBOL *)TABLE_get_symbol(arch_table, ind);
 	sym->pos = get_pos();
 
 	file = fopen(path, "r");
 	if (file == NULL)
-		THROW("Cannot open file '&1'", path);
+		THROW("Cannot open file: &1", path);
 
 	fstat(fileno(file), &info);
 
@@ -316,7 +368,7 @@ PUBLIC int ARCH_add_file(const char *path)
 		sym->len = 0;
 		fclose(file);
 		if (ARCH_verbose)
-			printf("Adding directory %s", rel_path);
+			printf(" %8s", "-");
 	}
 	else
 	{
@@ -325,14 +377,14 @@ PUBLIC int ARCH_add_file(const char *path)
 		len = sym->len;
 		while (len > 0)
 		{
-			len_read = fread(arch_buffer, 1, sizeof(arch_buffer), file);
+			len_read = fread(arch_buffer, 1, ARCH_BUFFER_SIZE, file);
 			if (len_read > 0)
 				fwrite(arch_buffer, 1, len_read, arch_file);
 
-			if (len_read < sizeof(arch_buffer))
+			if (len_read < ARCH_BUFFER_SIZE)
 			{
 				if (ferror(file))
-					THROW("Cannot read file '&1': &2", path, strerror(errno));
+					THROW("Cannot read file: &1: &2", path, strerror(errno));
 				else
 					break;
 			}
@@ -341,11 +393,52 @@ PUBLIC int ARCH_add_file(const char *path)
 		fclose(file);
 
 		if (ARCH_verbose)
-			printf("Adding file %s (%d bytes)", rel_path, sym->len);
+			printf(" %8d", sym->len);
 	}
 	
 	if (ARCH_verbose)
-		printf(" -> %d\n", ind);
+		printf(" %5d\n", ind);
 	
 	return ind;
 }
+
+#if 0
+void ARCH_browse(ARCH *a, void (*found)(const char *path, int64_t size))
+{
+	int i;
+	ARCH_SYMBOL *asym;
+	SYMBOL *sym;
+	char *path;
+	char *temp;
+	int size;
+	int ip;
+	
+	for (i = 0; i < a->header.n_symbol; i++)
+	{
+		asym = &a->symbol[i];
+		sym = &asym->sym;
+		
+		size = asym->len;
+		
+		path = STR_copy_len(sym->name, sym->len);
+		for(;;)
+		{
+			if (*path != '/')
+				break;
+			
+			ip = atoi(&path[1]);
+			sym = &a->symbol[ip].sym;
+			
+			temp = path;
+			path = STR_copy_len(sym->name, sym->len);
+			if (path[sym->len - 1] != '/')
+				STRING_add(&path, "/", 1);
+			STRING_add(&path, strchr(temp, ':') + 1, 0);
+			STRING_free(&temp);
+		}
+		
+		(*found)(path, size);
+		STR_free(path);
+	}
+}
+#endif
\ No newline at end of file
