@@ -1,14 +1,12 @@
 /***************************************************************************
 
-  stream_memory.c
+  gbx_stream_memory.c
 
-  The memory stream management routines
-
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -42,44 +41,20 @@
 #include "gbx_stream.h"
 
 
-#define FD (stream->buffer.file)
-
 static int stream_open(STREAM *stream, const char *path, int mode)
 {
-  /*
-  int fd;
-  struct stat info;
-  */
-
-  /*
-  if (path != NULL)
-  {
-    if ((mode & ST_MODE) != ST_READ)
-      return TRUE;
-
-    fd = open(path, O_RDONLY);
-    if (fd < 0)
-      return TRUE;
-
-    if (fstat(fd, &info) < 0)
-      return TRUE;
-    else
-      stream->memory.size = info.st_size;
-
-    stream->memory.addr = mmap(NULL, stream->memory.size, PROT_READ, MAP_static, fd, (off_t)0);
-    close(fd);
-    if (stream->memory.addr == MAP_FAILED)
-      return TRUE;
-  }
-  else
-  */
-  {
-    if (stream->memory.addr == NULL)
-      return TRUE;
-  }
+	stream->memory.addr = (char *)path;
 
+	if (stream->memory.addr == NULL)
+	{
+		stream->type = NULL;
+		THROW(E_ARG);
+	}
+	
   stream->memory.pos = 0;
 
+	stream->common.available_now = TRUE;
+
   return FALSE;
 }
 
@@ -97,9 +72,12 @@ static int stream_close(STREAM *stream)
 
 static int stream_read(STREAM *stream, char *buffer, int len)
 {
+	/*if ((stream->common.mode & STO_READ) == 0)
+		THROW(E_ACCESS);*/
+	
   CHECK_enter();
   
-  if (setjmp(CHECK_jump) == 0)
+  if (sigsetjmp(CHECK_jump, TRUE) == 0)
     memmove(buffer, stream->memory.addr + stream->memory.pos, len);
 
   CHECK_leave();  
@@ -107,23 +85,23 @@ static int stream_read(STREAM *stream, char *buffer, int len)
   if (CHECK_got_error())
   {
     errno = EIO;  
-    return TRUE;
+    return -1;
   }
   else
   {
     stream->memory.pos += len;
-    STREAM_eff_read = len;
-    return FALSE;
+    return len;
   }
 }
 
-#define stream_getchar NULL
-
 static int stream_write(STREAM *stream, char *buffer, int len)
 {
+	if ((stream->common.mode & STO_WRITE) == 0)
+		THROW(E_ACCESS);
+	
   CHECK_enter();
   
-  if (setjmp(CHECK_jump) == 0)
+  if (sigsetjmp(CHECK_jump, TRUE) == 0)
     memmove(stream->memory.addr + stream->memory.pos, buffer, len);
 
   CHECK_leave();  
@@ -131,12 +109,12 @@ static int stream_write(STREAM *stream, char *buffer, int len)
   if (CHECK_got_error())
   {
     errno = EIO;  
-    return TRUE;
+    return -1;
   }
   else
   {
     stream->memory.pos += len;
-    return FALSE;
+    return len;
   }
 }
 
@@ -172,7 +150,7 @@ static int stream_seek(STREAM *stream, int64_t pos, int whence)
 
 static int stream_tell(STREAM *stream, int64_t *pos)
 {
-  *pos = stream->memory.pos;
+  *pos = (int64_t)stream->memory.pos;
   return FALSE;
 }
 
