@@ -1,14 +1,12 @@
 /***************************************************************************
 
-  subr_test.c
+  gbx_subr_test.c
 
-  Test and Boolean routines
-
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -39,7 +38,7 @@
 #include "gbx_subr_test_temp.h"
 
 
-void SUBR_bit(void)
+void SUBR_bit(ushort code)
 {
   static void *jump[16] = {
     &&__ERROR, &&__BCLR, &&__BSET, &&__BTST, &&__BCHG, &&__ASL, &&__ASR, &&__ROL,
@@ -70,7 +69,7 @@ void SUBR_bit(void)
 
 	n = nbits[type];
 
-  VALUE_conv(&PARAM[1], T_INTEGER);
+  VALUE_conv_integer(&PARAM[1]);
   bit = PARAM[1]._integer.value;
 
   if ((bit < 0) || (bit >= n))
@@ -78,7 +77,7 @@ void SUBR_bit(void)
 
   RETURN->type = type;
 
-	goto *jump[EXEC_code & 0xF];
+	goto *jump[code & 0xF];
 
 __BCLR:
 
@@ -247,7 +246,7 @@ __LSR:
 
 __ERROR:
 
-	EXEC_ILLEGAL();
+	THROW_ILLEGAL();
 
 __END:
 
@@ -261,7 +260,7 @@ __END:
 		goto __END_VARIANT;
   	
 	__END_SHORT:
-		RETURN->_integer.value = (int)val & 0xFFFF;
+		RETURN->_integer.value = (int)(short)val;
 		goto __END_VARIANT;
 	
 	__END_INTEGER:
@@ -276,146 +275,67 @@ __END:
 __END_VARIANT:
 
 	if (variant)
-		VALUE_conv(RETURN, T_VARIANT);
+		VALUE_conv_variant(RETURN);
 
 __LEAVE:
 
   SUBR_LEAVE();
 }
 
-#if 0
-void SUBR_min_max(void)
-{
-  TYPE type = T_VOID;
-  int i;
-  boolean is_max;
 
-  SUBR_ENTER();
-
-  is_max = ((EXEC_code >> 8) == CODE_MAX);
-
-  for (i = 0; i < NPARAM; i++)
-  {
-    VARIANT_undo(&PARAM[i]);
-    type = Max(type, PARAM[i].type);
-  }
-
-  if (!TYPE_is_number_date(type))
-    THROW(E_TYPE, "Number or Date", TYPE_get_name(type));
-
-  VALUE_conv(PARAM, type);
-
-  switch (type)
-  {
-    case T_FLOAT:
-
-      for (i = 1; i < NPARAM; i++)
-      {
-        VALUE_conv(&PARAM[i], type);
-        if ((PARAM[i]._float.value < PARAM[0]._float.value) ^ is_max)
-          PARAM[0]._float.value = PARAM[i]._float.value;
-      }
-      break;
-
-    case T_DATE:
-
-      for (i = 1; i < NPARAM; i++)
-      {
-        VALUE_conv(&PARAM[i], type);
-        if (DATE_comp_value(&PARAM[i], &PARAM[0]) == (is_max ? 1 : -1))
-        {
-          PARAM[0]._date.date = PARAM[i]._date.date;
-          PARAM[0]._date.time = PARAM[i]._date.time;
-        }
-      }
-      break;
-
-    case T_LONG:
-
-      for (i = 1; i < NPARAM; i++)
-      {
-        VALUE_conv(&PARAM[i], type);
-        if ((PARAM[i]._long.value < PARAM[0]._long.value) ^ is_max)
-          PARAM[0]._long.value = PARAM[i]._long.value;
-      }
-      break;
-
-    default:
-
-      for (i = 1; i < NPARAM; i++)
-      {
-        VALUE_conv(&PARAM[i], type);
-        if ((PARAM[i]._integer.value < PARAM[0]._integer.value) ^ is_max)
-          PARAM[0]._integer.value = PARAM[i]._integer.value;
-      }
-      break;
-  }
-
-  SP -= NPARAM;
-  SP++;
-}
-#endif
-
-void SUBR_if(void)
+void SUBR_if(ushort code)
 {
+	int i;
+	unsigned char test;
+	TYPE type;
+	
   SUBR_ENTER_PARAM(3);
 
-  VALUE_conv(PARAM, T_BOOLEAN);
+  VALUE_conv_boolean(PARAM);
+	i = PARAM->_boolean.value ? 1 : 2;
+	
+	test = code & 0x1F;
 
-	switch (EXEC_code & 0x1F)
+	if (!test)
 	{
-		case 0:
-		
-			if (PARAM[1].type == PARAM[2].type)
-			{
-				*PC |= 1;
-				goto __TYPE;
-			}
-			else 
-			{
-				*PC |= 2;
-				goto __VARIANT;
-			}
-		
-		case 1:
-
-		__TYPE:
-			*RETURN = PARAM->_boolean.value ? PARAM[1] : PARAM[2];
+		type = PARAM[1].type;
+		if (PARAM[2].type == type && type <= T_VARIANT)
+		{
+			*PC |= 0x1F;
+		}
+		else
+		{
+			type = SUBR_check_good_type(&PARAM[1], 2);
+			if (TYPE_is_object(type))
+				type = T_OBJECT;
+			*PC |= (unsigned char)type;
 			
-			break;
-			
-		case 2:
-		__VARIANT:
-		
-			if (PARAM->_boolean.value)
-			{
-				VALUE_conv(&PARAM[1], T_VARIANT);
-				*RETURN = PARAM[1];
-			}
-			else
-			{
-				VALUE_conv(&PARAM[2], T_VARIANT);
-				*RETURN = PARAM[2];
-			}
-			break;
+			VALUE_conv(&PARAM[i], type);
+		}
+	}
+	else if (test != 0x1F)
+	{
+		VALUE_conv(&PARAM[i], (TYPE)test);
 	}
 
-  SUBR_LEAVE();
+	*PARAM = PARAM[i];
+	RELEASE(&PARAM[3 - i]);
+	SP -= 2;
 }
 
 
-void SUBR_choose(void)
+void SUBR_choose(ushort code)
 {
   int val;
 
   SUBR_ENTER();
 
-  VALUE_conv(PARAM, T_INTEGER);
+  VALUE_conv_integer(PARAM);
   val = PARAM->_integer.value;
 
-  if (val >= 1 && val <= NPARAM)
+  if (val >= 1 && val < NPARAM)
   {
-    VALUE_conv(&PARAM[val], T_VARIANT);
+    VALUE_conv_variant(&PARAM[val]);
     *RETURN = PARAM[val];
   }
   else
@@ -434,13 +354,14 @@ void SUBR_near(void)
 
   SUBR_ENTER_PARAM(2);
 
-  VALUE_conv(&PARAM[0], T_STRING);
-  VALUE_conv(&PARAM[1], T_STRING);
+  VALUE_conv_string(&PARAM[0]);
+  VALUE_conv_string(&PARAM[1]);
 
-  result = STRING_comp_value_ignore_case(&PARAM[0], &PARAM[1]) ? 0 : -1;
+  //result = STRING_comp_value_ignore_case(&PARAM[0], &PARAM[1]) ? -1 : 0;
+  result = STRING_equal_ignore_case(PARAM[0]._string.addr + PARAM[0]._string.start, PARAM[0]._string.len, PARAM[1]._string.addr + PARAM[1]._string.start, PARAM[1]._string.len) ? -1 : 0;
 
-  RELEASE(&PARAM[0]);
-  RELEASE(&PARAM[1]);
+  RELEASE_STRING(&PARAM[0]);
+  RELEASE_STRING(&PARAM[1]);
 
   PARAM->type = T_BOOLEAN;
   PARAM->_boolean.value = result;
@@ -448,221 +369,498 @@ void SUBR_near(void)
   SP--;
 }
 
-
-void SUBR_comp(void)
+#if 0
+void SUBR_comp(ushort code)
 {
-  static void *jump[17] = {
-    &&__VARIANT, &&__BOOLEAN, &&__BYTE, &&__SHORT, &&__INTEGER, &&__LONG, &&__SINGLE, &&__FLOAT, &&__DATE,
-    &&__STRING, &&__STRING, &&__ERROR, &&__ERROR, &&__ERROR, &&__ERROR,
-    &&__NULL, &&__OBJECT
-    };
+	static void *jump[17] = {
+		&&__VARIANT, &&__BOOLEAN, &&__BYTE, &&__SHORT, &&__INTEGER, &&__LONG, &&__SINGLE, &&__FLOAT, &&__DATE,
+		&&__STRING, &&__STRING, &&__POINTER, &&__ERROR, &&__ERROR, &&__ERROR, &&__NULL, &&__OBJECT
+		};
 
-  static void *test[] = { &&__EQ, &&__NE, &&__GT, &&__LE, &&__LT, &&__GE };
+	//static void *test[] = { &&__EQ, &&__NE, &&__GT, &&__LE, &&__LT, &&__GE };
 
-  char result = 0; // prevents a gcc warning
-  char op;
-  VALUE *P1;
-  VALUE *P2;
-  bool variant;
-  TYPE type, typem;
+	char NO_WARNING(result);
+	VALUE *P1;
+	VALUE *P2;
 
-  P1 = SP - 2;
-  P2 = P1 + 1;
+	P1 = SP - 2;
+	P2 = P1 + 1;
 
-  type = EXEC_code & 0x1F;
-  op = (EXEC_code - C_EQ) >> 8;
-  goto *jump[type];
+	goto *jump[code & 0x1F];
 
 __BOOLEAN:
 __BYTE:
 __SHORT:
 __INTEGER:
 
-  {
-    int tmp = P1->_integer.value - P2->_integer.value;
-    result = tmp > 0 ? 1 : tmp < 0 ? -1 : 0;
-    goto __END;
-  }
-  
+	result = P1->_integer.value == P2->_integer.value;
+	goto __END;
+	
 __LONG:
 
-  VALUE_conv(P1, T_LONG);
-  VALUE_conv(P2, T_LONG);
+	VALUE_conv(P1, T_LONG);
+	VALUE_conv(P2, T_LONG);
 
-  result = llsgn(P1->_long.value - P2->_long.value);
-  goto __END;
+	result = P1->_long.value == P2->_long.value;
+	goto __END;
 
 __DATE:
 
-  VALUE_conv(P1, T_DATE);
-  VALUE_conv(P2, T_DATE);
+	VALUE_conv(P1, T_DATE);
+	VALUE_conv(P2, T_DATE);
 
-  result = DATE_comp_value(P1, P2);
-  goto __END;
+	result = DATE_comp_value(P1, P2) == 0;
+	goto __END;
 
 __NULL:
 
-  if (op < 2)
-  {
-  	if (P2->type == T_NULL)
-  	{
-	    result = !VALUE_is_null(P1);
-	    goto __END_RELEASE;
-		}
-		else if (P1->type == T_NULL)
-		{
-	    result = !VALUE_is_null(P2);
-	    goto __END_RELEASE;
-		}
-  }
+	if (P2->type == T_NULL)
+	{
+		result = VALUE_is_null(P1);
+		goto __END_RELEASE;
+	}
+	else if (P1->type == T_NULL)
+	{
+		result = VALUE_is_null(P2);
+		goto __END_RELEASE;
+	}
 
 __STRING:
 
-  VALUE_conv(P1, T_STRING);
-  VALUE_conv(P2, T_STRING);
+	VALUE_conv_string(P1);
+	VALUE_conv_string(P2);
 
-  if (op < 2 && P1->_string.len != P2->_string.len)
-    result = 1;
-  else
-    result = STRING_comp_value(P1, P2);
-  goto __END_RELEASE;
+	if (P1->_string.len != P2->_string.len)
+		result = 0;
+	else
+		result = STRING_equal_same(P1->_string.addr + P1->_string.start, P2->_string.addr + P2->_string.start, P1->_string.len);
+	
+	RELEASE_STRING(P1);
+	RELEASE_STRING(P2);
+	goto __END;
 
 __SINGLE:
 __FLOAT:
 
-  VALUE_conv(P1, T_FLOAT);
-  VALUE_conv(P2, T_FLOAT);
+	VALUE_conv_float(P1);
+	VALUE_conv_float(P2);
 
-  result = fsgn(P1->_float.value - P2->_float.value);
-  goto __END;
+	result = P1->_float.value == P2->_float.value;
+	goto __END;
+
+__POINTER:
+
+	VALUE_conv(P1, T_POINTER);
+	VALUE_conv(P2, T_POINTER);
+
+	result = P1->_pointer.value == P2->_pointer.value;
+	goto __END;
 
 __OBJECT:
 
-  result = OBJECT_comp_value(P1, P2);
-  goto __END_RELEASE;
+	result = OBJECT_comp_value(P1, P2) == 0;
+	//RELEASE_OBJECT(P1);
+	//RELEASE_OBJECT(P2);
+	goto __END_RELEASE;
 
 __VARIANT:
 
-  variant = FALSE;
-  
-  if (TYPE_is_variant(P1->type))
-  {
-    VARIANT_undo(P1);
-    variant = TRUE;
-  }
+	{
+		bool variant = FALSE;
+		TYPE type;
+	
+		if (TYPE_is_variant(P1->type))
+		{
+			VARIANT_undo(P1);
+			variant = TRUE;
+		}
 
-  if (TYPE_is_variant(P2->type))
-  {
-    VARIANT_undo(P2);
-    variant = TRUE;
-  }
+		if (TYPE_is_variant(P2->type))
+		{
+			VARIANT_undo(P2);
+			variant = TRUE;
+		}
 
-  type = Max(P1->type, P2->type);
+		type = Max(P1->type, P2->type);
 
-  if (op > 1)
-  {
-    if (type == T_NULL || TYPE_is_string(type))
-    {
-      typem = Min(P1->type, P2->type);
-      if (!TYPE_is_string(typem))
-        THROW(E_TYPE, TYPE_get_name(typem), TYPE_get_name(type));
-    }
-    else if (TYPE_is_object(type))
-      goto __ERROR;
-  }
-  else
-  {
-    if (TYPE_is_object_null(P1->type) && TYPE_is_object_null(P2->type))
-      type = T_OBJECT;
-    else if (TYPE_is_object(type))
-      THROW(E_TYPE, "Object", TYPE_get_name(Min(P1->type, P2->type)));
-  }
+		if (TYPE_is_object_null(P1->type) && TYPE_is_object_null(P2->type))
+			type = T_OBJECT;
+		else if (TYPE_is_object(type))
+			THROW(E_TYPE, "Object", TYPE_get_name(Min(P1->type, P2->type)));
 
-  if (!variant)
-    *PC |= type;
+		if (!variant)
+			*PC |= type;
 
-  goto *jump[type];
+		goto *jump[type];
+	}
 
 __ERROR:
 
-  THROW(E_TYPE, "Number, Date or String", TYPE_get_name(type));
+	THROW(E_TYPE, "Number, Date or String", TYPE_get_name(code & 0x1F));
 
 __END_RELEASE:
 
-  RELEASE(P1);
-  RELEASE(P2);
+	RELEASE(P1);
+	RELEASE(P2);
 
 __END:
 
-  P1->type = T_BOOLEAN;
-  SP--;
-
-  goto *test[(int)op];
-
-__EQ:
-  P1->_boolean.value = result == 0 ? -1 : 0;
-  return;
-
-__NE:
-  P1->_boolean.value = result != 0 ? -1 : 0;
-  return;
-
-__GT:
-  P1->_boolean.value = result > 0 ? -1 : 0;
-  return;
-
-__GE:
-  P1->_boolean.value = result >= 0 ? -1 : 0;
-  return;
-
-__LT:
-  P1->_boolean.value = result < 0 ? -1 : 0;
-  return;
-
-__LE:
-  P1->_boolean.value = result <= 0 ? -1 : 0;
-  return;
+	P1->type = T_BOOLEAN;
+	SP--;
+	P1->_boolean.value = -result;
 }
 
+void SUBR_compn(ushort code)
+{
+	static void *jump[17] = {
+		&&__VARIANT, &&__BOOLEAN, &&__BYTE, &&__SHORT, &&__INTEGER, &&__LONG, &&__SINGLE, &&__FLOAT, &&__DATE,
+		&&__STRING, &&__STRING, &&__POINTER, &&__ERROR, &&__ERROR, &&__ERROR, &&__NULL, &&__OBJECT
+		};
 
+	//static void *test[] = { &&__EQ, &&__NE, &&__GT, &&__LE, &&__LT, &&__GE };
 
-void SUBR_strcomp(void)
+	char NO_WARNING(result);
+	VALUE *P1;
+	VALUE *P2;
+
+	P1 = SP - 2;
+	P2 = P1 + 1;
+
+	goto *jump[code & 0x1F];
+
+__BOOLEAN:
+__BYTE:
+__SHORT:
+__INTEGER:
+
+	result = P1->_integer.value == P2->_integer.value;
+	goto __END;
+	
+__LONG:
+
+	VALUE_conv(P1, T_LONG);
+	VALUE_conv(P2, T_LONG);
+
+	result = P1->_long.value == P2->_long.value;
+	goto __END;
+
+__DATE:
+
+	VALUE_conv(P1, T_DATE);
+	VALUE_conv(P2, T_DATE);
+
+	result = DATE_comp_value(P1, P2) == 0;
+	goto __END;
+
+__NULL:
+
+	if (P2->type == T_NULL)
+	{
+		result = VALUE_is_null(P1);
+		goto __END_RELEASE;
+	}
+	else if (P1->type == T_NULL)
+	{
+		result = VALUE_is_null(P2);
+		goto __END_RELEASE;
+	}
+
+__STRING:
+
+	VALUE_conv_string(P1);
+	VALUE_conv_string(P2);
+
+	if (P1->_string.len != P2->_string.len)
+		result = 0;
+	else
+		result = STRING_equal_same(P1->_string.addr + P1->_string.start, P2->_string.addr + P2->_string.start, P1->_string.len);
+	
+	RELEASE_STRING(P1);
+	RELEASE_STRING(P2);
+	goto __END;
+
+__SINGLE:
+__FLOAT:
+
+	VALUE_conv_float(P1);
+	VALUE_conv_float(P2);
+
+	result = P1->_float.value == P2->_float.value;
+	goto __END;
+
+__POINTER:
+
+	VALUE_conv(P1, T_POINTER);
+	VALUE_conv(P2, T_POINTER);
+
+	result = P1->_pointer.value == P2->_pointer.value;
+	goto __END;
+
+__OBJECT:
+
+	result = OBJECT_comp_value(P1, P2) == 0;
+	//RELEASE_OBJECT(P1);
+	//RELEASE_OBJECT(P2);
+	goto __END_RELEASE;
+
+__VARIANT:
+
+	{
+		bool variant = FALSE;
+		TYPE type;
+	
+		if (TYPE_is_variant(P1->type))
+		{
+			VARIANT_undo(P1);
+			variant = TRUE;
+		}
+
+		if (TYPE_is_variant(P2->type))
+		{
+			VARIANT_undo(P2);
+			variant = TRUE;
+		}
+
+		type = Max(P1->type, P2->type);
+
+		if (TYPE_is_object_null(P1->type) && TYPE_is_object_null(P2->type))
+			type = T_OBJECT;
+		else if (TYPE_is_object(type))
+			THROW(E_TYPE, "Object", TYPE_get_name(Min(P1->type, P2->type)));
+
+		if (!variant)
+			*PC |= type;
+
+		goto *jump[type];
+	}
+
+__ERROR:
+
+	THROW(E_TYPE, "Number, Date or String", TYPE_get_name(code & 0x1F));
+
+__END_RELEASE:
+
+	RELEASE(P1);
+	RELEASE(P2);
+
+__END:
+
+	P1->type = T_BOOLEAN;
+	SP--;
+
+	P1->_boolean.value = result - 1; // ? 0 : -1;
+}
+
+#define sgn(_x) \
+({ \
+	int x = _x; \
+	int minusOne = x >> 31; \
+	unsigned int negateX = (unsigned int) -x; \
+	int plusOne = (int)(negateX >> 31); \
+  int result = minusOne | plusOne; \
+  result; \
+})
+
+void SUBR_compi(ushort code)
+{
+	static void *jump[17] = {
+		&&__VARIANT, &&__BOOLEAN, &&__BYTE, &&__SHORT, &&__INTEGER, &&__LONG, &&__SINGLE, &&__FLOAT, &&__DATE,
+		&&__STRING, &&__STRING, &&__POINTER, &&__ERROR, &&__ERROR, &&__ERROR, &&__NULL, &&__OBJECT
+		};
+
+	static void *test[] = { &&__GT, &&__LE, &&__LT, &&__GE };
+
+	char NO_WARNING(result);
+	VALUE *P1;
+	VALUE *P2;
+	TYPE type;
+
+	P1 = SP - 2;
+	P2 = P1 + 1;
+
+	type = code & 0x1F;
+	goto *jump[type];
+
+__BOOLEAN:
+__BYTE:
+__SHORT:
+__INTEGER:
+
+	result = P1->_integer.value > P2->_integer.value ? 1 : P1->_integer.value < P2->_integer.value ? -1 : 0;
+	goto __END;
+	
+__LONG:
+
+	VALUE_conv(P1, T_LONG);
+	VALUE_conv(P2, T_LONG);
+
+	result = P1->_long.value > P2->_long.value ? 1 : P1->_long.value < P2->_long.value ? -1 : 0;
+	goto __END;
+
+__DATE:
+
+	VALUE_conv(P1, T_DATE);
+	VALUE_conv(P2, T_DATE);
+
+	result = DATE_comp_value(P1, P2);
+	goto __END;
+
+__NULL:
+__STRING:
+
+	VALUE_conv_string(P1);
+	VALUE_conv_string(P2);
+
+	result = STRING_compare(P1->_string.addr + P1->_string.start, P1->_string.len, P2->_string.addr + P2->_string.start, P2->_string.len);
+	
+	RELEASE_STRING(P1);
+	RELEASE_STRING(P2);
+	goto __END;
+
+__SINGLE:
+
+	VALUE_conv(P1, T_SINGLE);
+	VALUE_conv(P2, T_SINGLE);
+
+	result = P1->_single.value > P2->_single.value ? 1 : P1->_single.value < P2->_single.value ? -1 : 0;
+	goto __END;
+
+__FLOAT:
+
+	VALUE_conv_float(P1);
+	VALUE_conv_float(P2);
+
+	result = P1->_float.value > P2->_float.value ? 1 : P1->_float.value < P2->_float.value ? -1 : 0;
+	goto __END;
+
+__POINTER:
+
+	VALUE_conv(P1, T_POINTER);
+	VALUE_conv(P2, T_POINTER);
+
+	result = P1->_pointer.value > P2->_pointer.value ? 1 : P1->_pointer.value < P2->_pointer.value ? -1 : 0;
+	goto __END;
+
+__OBJECT:
+
+	result = OBJECT_comp_value(P1, P2);
+	//RELEASE_OBJECT(P1);
+	//RELEASE_OBJECT(P2);
+	goto __END_RELEASE;
+
+__VARIANT:
+
+	{
+		bool variant = FALSE;
+	
+		if (TYPE_is_variant(P1->type))
+		{
+			VARIANT_undo(P1);
+			variant = TRUE;
+		}
+
+		if (TYPE_is_variant(P2->type))
+		{
+			VARIANT_undo(P2);
+			variant = TRUE;
+		}
+
+		type = Max(P1->type, P2->type);
+
+		if (type == T_NULL || TYPE_is_string(type))
+		{
+			TYPE typem = Min(P1->type, P2->type);
+			if (!TYPE_is_string(typem))
+				THROW(E_TYPE, TYPE_get_name(typem), TYPE_get_name(type));
+		}
+		else if (TYPE_is_object(type))
+			goto __ERROR;
+
+		if (!variant)
+			*PC |= type;
+
+		goto *jump[type];
+	}
+
+__ERROR:
+
+	THROW(E_TYPE, "Number, Date or String", TYPE_get_name(type));
+
+__END_RELEASE:
+
+	RELEASE(P1);
+	RELEASE(P2);
+
+__END:
+
+	P1->type = T_BOOLEAN;
+	SP--;
+
+	goto *test[(code >> 8) - (C_GT >> 8)];
+
+__GT:
+	P1->_boolean.value = result > 0 ? -1 : 0;
+	return;
+
+__GE:
+	P1->_boolean.value = result >= 0 ? -1 : 0;
+	return;
+
+__LT:
+	P1->_boolean.value = result < 0 ? -1 : 0;
+	return;
+
+__LE:
+	P1->_boolean.value = result <= 0 ? -1 : 0;
+	return;
+}
+#endif
+
+void SUBR_strcomp(ushort code)
 {
   int mode = GB_COMP_BINARY;
-  char *s1, *s2;
+  /*char *s1, *s2;
   int l1, l2;
-  int ret;
+  int ret;*/
 
   SUBR_ENTER();
 
-  VALUE_conv(&PARAM[0], T_STRING);
-  VALUE_conv(&PARAM[1], T_STRING);
+  VALUE_conv_string(&PARAM[0]);
+  VALUE_conv_string(&PARAM[1]);
 
   if (NPARAM == 3)
     mode = SUBR_get_integer(&PARAM[2]);
 
-	mode &= GB_COMP_TYPE_MASK;
+	/*mode &= GB_COMP_TYPE_MASK;
 	
 	if (mode == GB_COMP_BINARY)
-		ret = STRING_comp_value(&PARAM[0], &PARAM[1]);
-	else if (mode == GB_COMP_TEXT)
-		ret = STRING_comp_value_ignore_case(&PARAM[0], &PARAM[1]);
+		ret = STRING_compare(PARAM[0]._string.addr + PARAM[0]._string.start, PARAM[0]._string.len, PARAM[1]._string.addr + PARAM[1]._string.start, PARAM[1]._string.len);
+	else if (mode == GB_COMP_NOCASE)
+		ret = STRING_compare_ignore_case(PARAM[0]._string.addr + PARAM[0]._string.start, PARAM[0]._string.len, PARAM[1]._string.addr + PARAM[1]._string.start, PARAM[1]._string.len);
 	else
 	{
 		SUBR_get_string_len(&PARAM[0], &s1, &l1);
 		SUBR_get_string_len(&PARAM[1], &s2, &l2);
-	
-		ret = COMPARE_string_lang(s1, l1, s2, l2, mode, FALSE);
-  }
+		
+		if (mode & GB_COMP_NATURAL)
+			ret = COMPARE_string_natural(s1, l1, s2, l2, mode & GB_COMP_NOCASE);
+		else if (mode & GB_COMP_LIKE)
+			ret = COMPARE_string_like(s1, l1, s2, l2, mode & GB_COMP_NOCASE);
+		else if (mode & GB_COMP_LANG)
+			ret = COMPARE_string_lang(s1, l1, s2, l2, mode & GB_COMP_NOCASE, FALSE);
+		else
+			THROW(E_ARG);
+	}*/
   
 	RETURN->_integer.type = T_INTEGER;
-	RETURN->_integer.value = ret;
+	RETURN->_integer.value = (*COMPARE_get_string_func(mode))(PARAM[0]._string.addr + PARAM[0]._string.start, PARAM[0]._string.len, PARAM[1]._string.addr + PARAM[1]._string.start, PARAM[1]._string.len, mode & GB_COMP_NOCASE, FALSE);
 
   SUBR_LEAVE();
 }
 
 
-void SUBR_is(void)
+
+void SUBR_is(ushort code)
 {
 	VALUE *P1 = SP - 2;
 	VALUE *P2 = SP - 1;
@@ -679,15 +877,15 @@ void SUBR_is(void)
 	else
 		res = (OBJECT_class(object) == klass || CLASS_inherits(OBJECT_class(object), klass));
 
-	OBJECT_UNREF(object, "SUBR_is");
+	OBJECT_UNREF(object);
 
 	P1->type = T_BOOLEAN;
-	P1->_boolean.value = res ? -1 : 0;
+	P1->_boolean.value = -(res ^ (code & 1));
 	SP--;
 }
 
 
-void SUBR_min_max(void)
+void SUBR_min_max(ushort code)
 {
   static void *jump[] = {
     &&__VARIANT, &&__BOOLEAN, &&__BYTE, &&__SHORT, &&__INTEGER, &&__LONG, &&__FLOAT, &&__FLOAT, &&__DATE
@@ -702,8 +900,8 @@ void SUBR_min_max(void)
   P2 = P1 + 1;
 
   jump_end = &&__END;
-  type = EXEC_code & 0x0F;
-  is_max = ((EXEC_code >> 8) == CODE_MAX);
+  type = code & 0x0F;
+  is_max = ((code >> 8) == CODE_MAX);
   goto *jump[type];
 
 __BOOLEAN:
@@ -746,8 +944,8 @@ __LONG:
 
 __FLOAT:
 
-  VALUE_conv(P1, T_FLOAT);
-  VALUE_conv(P2, T_FLOAT);
+  VALUE_conv_float(P1);
+  VALUE_conv_float(P2);
 
 	if (is_max)
 	{
@@ -804,7 +1002,7 @@ __ERROR:
 
 __VARIANT_END:
 
-  VALUE_conv(P1, T_VARIANT);
+  VALUE_conv_variant(P1);
 
 __END:
 
