@@ -1,14 +1,12 @@
 /***************************************************************************
 
-  watch.c
+  gbx_watch.c
 
-  Default event loop and file descriptor watch routines
-
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -38,14 +37,15 @@
 #include "gbx_c_timer.h"
 #include "gbx_watch.h"
 
-
-//#define DEBUG_TIMER
-//#define DEBUG_WATCH
+//#define DEBUG_TIMER 1
+//#define DEBUG_WATCH 1
 
 static fd_set read_fd;
 static fd_set write_fd;
 
 static WATCH_CALLBACK *watch_callback = NULL;
+//static short *watch_index = NULL;
+
 static int max_fd = 0;
 
 static WATCH_TIMER *_timers = NULL;
@@ -53,6 +53,8 @@ static WATCH_TIMER *_timers = NULL;
 static int _do_not_really_delete_callback = 0;
 static bool _must_delete_callback = FALSE;
 
+static struct timeval _last_time = { 0 };
+
 #ifdef DEBUG_TIMER
 double _debug_time;
 static double time_to_double(const struct timeval *t)
@@ -96,17 +98,17 @@ static struct timeval *time_now(void)
 {
 	static struct timeval current;
 
-  gettimeofday(&current, NULL);
-  if (current.tv_usec < 0 || current.tv_usec >= 1000000)
-  	fprintf(stderr, "gettimeofday: tv_usec = %ld!\n", current.tv_usec);
-  return &current;
+	gettimeofday(&current, NULL);
+	if (current.tv_usec < 0 || current.tv_usec >= 1000000)
+		fprintf(stderr, "gbx3: warning: gettimeofday: tv_usec = %ld!\n", current.tv_usec);
+	return &current;
 }
 
 
 static void add_timer(GB_TIMER *timer, const struct timeval *timeout)
 {
-  int i;
-  WATCH_TIMER *wt;
+	int i;
+	WATCH_TIMER *wt;
 
 	for (i = 0; i < ARRAY_count(_timers); i++)
 	{
@@ -124,22 +126,40 @@ static void add_timer(GB_TIMER *timer, const struct timeval *timeout)
 	#endif
 }
 
-
-static void remove_timer(GB_TIMER *timer)
+static int find_timer(GB_TIMER *timer)
 {
-  int i;
+	int i;
 
 	for (i = 0; i < ARRAY_count(_timers); i++)
 	{
 		if (_timers[i].timer == timer)
-		{
-			ARRAY_remove(&_timers, i)
-			#ifdef DEBUG_TIMER
-			fprintf(stderr, "remove_timer: %p at %i\n", timer, i);
-			#endif
-			break;
-		}
+			return i;
 	}
+	
+	return (-1);
+}
+
+static void remove_timer(GB_TIMER *timer)
+{
+	int i = find_timer(timer);
+
+	if (i >= 0)
+	{
+		ARRAY_remove(&_timers, i)
+		#ifdef DEBUG_TIMER
+		fprintf(stderr, "remove_timer: %p at %i\n", timer, i);
+		#endif
+	}
+}
+
+double WATCH_get_timeout(GB_TIMER *timer)
+{
+	int i = find_timer(timer);
+
+	if (i < 0)
+		return 0.0;
+	else
+		return DATE_to_double(&_timers[i].timeout, TRUE);
 }
 
 static void raise_timers()
@@ -210,6 +230,9 @@ static bool get_timeout(const struct timeval *wait, struct timeval *tv)
 
 	if (timeout.tv_sec == 0 && timeout.tv_usec < (1000000 / 100))
 	{
+		#ifdef DEBUG_TIMER
+		fprintf(stderr, "busy loop\n");
+		#endif
 		// busy loop!
 		time_add(&timeout, now);
 		for(;;)
@@ -241,11 +264,12 @@ static bool get_timeout(const struct timeval *wait, struct timeval *tv)
 
 void WATCH_init(void)
 {
-  FD_ZERO(&read_fd);
-  FD_ZERO(&write_fd);
+	FD_ZERO(&read_fd);
+	FD_ZERO(&write_fd);
 
-  ARRAY_create(&watch_callback);
-  ARRAY_create(&_timers);
+	ARRAY_create(&watch_callback);
+	//ARRAY_create(&watch_index);
+	ARRAY_create(&_timers);
 
 	#ifdef DEBUG_TIMER
 	DATE_timer(&_debug_time, FALSE);
@@ -255,25 +279,26 @@ void WATCH_init(void)
 
 void WATCH_exit(void)
 {
-  ARRAY_delete(&watch_callback);
-  ARRAY_delete(&_timers);
+	ARRAY_delete(&watch_callback);
+	//ARRAY_delete(&watch_index);
+	ARRAY_delete(&_timers);
 }
 
 
 static void watch_fd(int fd, int flag, bool watch)
 {
 	#if DEBUG_WATCH
-  fprintf(stderr, "watch_fd: %d / %d: %d\n", fd, flag, watch);
+	fprintf(stderr, "watch_fd: %d for %s: %s\n", fd, flag == GB_WATCH_READ ? "read" : "write", watch ? "set" : "clear");
 	#endif
 
-  if (flag == WATCH_READ)
+	if (flag == GB_WATCH_READ)
 	{
 		if (watch)
 			FD_SET(fd, &read_fd);
 		else
 			FD_CLR(fd, &read_fd);
 	}
-	else if (flag == WATCH_WRITE)
+	else if (flag == GB_WATCH_WRITE)
 	{
 		if (watch)
 			FD_SET(fd, &write_fd);
@@ -285,97 +310,135 @@ static void watch_fd(int fd, int flag, bool watch)
 
 static int watch_find_callback(int fd)
 {
-  int i;
+	int i;
 
-  for (i = 0; i < ARRAY_count(watch_callback); i++)
-  {
-    if (fd == watch_callback[i].fd)
-      return i;
-  }
+	for (i = 0; i < ARRAY_count(watch_callback); i++)
+	{
+		if (watch_callback[i].fd == fd)
+			return i;
+	}
 
-  return (-1);
+	return -1;
 }
 
 
 static int find_max_fd(void)
 {
-  int i;
-  int max = -1;
+	int i;
+	int max = -1;
 
-  for (i = 0; i < ARRAY_count(watch_callback); i++)
-  {
-  	if (watch_callback[i].fd > max)
-  		max = watch_callback[i].fd;
-  }
+	for (i = 0; i < ARRAY_count(watch_callback); i++)
+	{
+		if (watch_callback[i].fd > max)
+			max = watch_callback[i].fd;
+	}
 
-  return max;
+	return max;
 }
 
+/*static void ensure_watch_index(int fd)
+{
+	int i;
+	int count = ARRAY_count(watch_index);
+	
+	//fprintf(stderr, "ensure_watch_index: %d (%d)\n", fd, count);
+	
+	if (fd < count)
+		return;
+	
+	ARRAY_add_many(&watch_index, fd - count + 1);
+	for (i = count; i <= fd; i++)
+		watch_index[i] = -1;
+}*/
 
 static WATCH_CALLBACK *watch_create_callback(int fd)
 {
-  int pos;
-  WATCH_CALLBACK *wcb;
+	int pos;
+	WATCH_CALLBACK *wcb;
 
 	#if DEBUG_WATCH
 	fprintf(stderr, "watch_create_callback: %d\n", fd);
 	#endif
 
-  pos = watch_find_callback(fd);
-  if (pos < 0)
-  {
-    wcb = ARRAY_add_void(&watch_callback);
-    wcb->fd = fd;
-  }
-  else
-    wcb = &watch_callback[pos];
+	pos = watch_find_callback(fd);
+	if (pos < 0)
+	{
+		pos = ARRAY_count(watch_callback);
+		wcb = ARRAY_add_void(&watch_callback);
+		wcb->fd = fd;
+	}
+	else
+		wcb = &watch_callback[pos];
 
 	if (fd > max_fd)
 		max_fd = fd;
 		
-  return wcb;
+	#if DEBUG_WATCH
+	fprintf(stderr, "watch_create_callback: %d -> %d read = %p (%p) write = %p (%p)\n", fd, pos, wcb->callback_read, (void *)wcb->param_read, wcb->callback_write, (void *)wcb->param_write);
+	#endif
+	
+	//ensure_watch_index(fd);
+	//watch_index[fd] = pos;
+	
+	return wcb;
 }
 
 
 static void watch_delete_callback(int fd)
 {
+	WATCH_CALLBACK *wcb;
 	int pos;
 	
+	#if DEBUG_WATCH
+	fprintf(stderr, "watch_delete_callback: %d (%d)\n", fd, _do_not_really_delete_callback);
+	#endif
+
 	pos = watch_find_callback(fd);
 	if (pos < 0)
 		return;
-		
-	watch_callback[pos].fd = -1;
-	watch_fd(fd, WATCH_READ, FALSE);
-	watch_fd(fd, WATCH_WRITE, FALSE);
+
+	//watch_index[fd] = -1;
+
+	wcb = &watch_callback[pos];
+	wcb->fd = -1;
+	watch_fd(fd, GB_WATCH_READ, FALSE);
+	watch_fd(fd, GB_WATCH_WRITE, FALSE);
 	max_fd = find_max_fd();
 	
 	if (_do_not_really_delete_callback)
 	{
+		#if DEBUG_WATCH
+		fprintf(stderr, "--> do not really delete\n");
+		#endif
 		_must_delete_callback = TRUE;
 		return;
 	}
 		
 	ARRAY_remove(&watch_callback, pos);
+
+	#if DEBUG_WATCH
+	fprintf(stderr, "--> deleted\n");
+	#endif
 }
 
 
 void WATCH_watch(int fd, int type, void *callback, intptr_t param)
 {
-  WATCH_CALLBACK *wcb;
+	WATCH_CALLBACK *wcb;
 
 	if (fd < 0 || fd > FD_SETSIZE)
 	{
-		fprintf(stderr, "WARNING: trying to watch fd %d\n", fd);
+		if (type != GB_WATCH_NONE)
+			ERROR_warning("trying to watch fd #%d", fd);
 		return;
 	}
 
-  if (type == WATCH_NONE)
-  	watch_delete_callback(fd);
-  else
-  {
-    wcb = watch_create_callback(fd);
-		if (type == WATCH_READ)
+	if (type == GB_WATCH_NONE)
+		watch_delete_callback(fd);
+	else
+	{
+		wcb = watch_create_callback(fd);
+		if (type == GB_WATCH_READ)
 		{
 			wcb->callback_read = callback;
 			wcb->param_read = param;
@@ -385,57 +448,81 @@ void WATCH_watch(int fd, int type, void *callback, intptr_t param)
 			wcb->callback_write = callback;
 			wcb->param_write = param;
 		}
+
+		#if DEBUG_WATCH
+		fprintf(stderr, "add watch: %d -> %d read = %p (%p) write = %p (%p)\n", fd, watch_find_callback(fd), wcb->callback_read, (void *)wcb->param_read, wcb->callback_write, (void *)wcb->param_write);
+		#endif
+
 		if (!wcb->callback_read && !wcb->callback_write)
 			watch_delete_callback(fd);
 		else
 		{
-			watch_fd(fd, WATCH_READ, wcb->callback_read != NULL);
-			watch_fd(fd, WATCH_WRITE, wcb->callback_write != NULL);
+			watch_fd(fd, GB_WATCH_READ, wcb->callback_read != NULL);
+			watch_fd(fd, GB_WATCH_WRITE, wcb->callback_write != NULL);
 		}
-		
-		#if DEBUG_WATCH
-    fprintf(stderr, "add watch: %d\n",  watch_find_callback(fd));
-		#endif
-  }
+	}
 }
 
 
 static void raise_callback(fd_set *rfd, fd_set *wfd)
 {
-  int i;
-  WATCH_CALLBACK wcb;
+	int i;
+	WATCH_CALLBACK wcb;
 
 	_do_not_really_delete_callback++;
 
 	#if DEBUG_WATCH
-	fprintf(stderr, "\nmax_fd = %d\n", max_fd);
+	fprintf(stderr, "raise_callback: max_fd = %d\n", max_fd);
 	#endif
-  for (i = 0; i < ARRAY_count(watch_callback); i++)
-  {
-  	// We copy the callback structure, because the watch_callback array can change during the
-  	// execution of the callbacks.
-  	
-    wcb = watch_callback[i];
-    if (wcb.fd < 0)
-    	continue;
-    
-    if (FD_ISSET(wcb.fd, rfd))
+	for (i = 0; i < ARRAY_count(watch_callback); i++)
+	{
+		// We copy the callback structure, because the watch_callback array can change during the
+		// execution of the callbacks.
+		
+		if (watch_callback[i].fd < 0)
+			continue;
+		
+		wcb = watch_callback[i];
+
+		#if DEBUG_WATCH
+		fprintf(stderr, "raise_callback: [%d] fd = %d read = %p (%p) write = %p (%p)\n", i, wcb.fd, wcb.callback_read, (void *)wcb.param_read, wcb.callback_write, (void *)wcb.param_write);
+		#endif
+		
+		if (FD_ISSET(wcb.fd, rfd))
 		{
+			FD_CLR(wcb.fd, rfd);
 			if (wcb.callback_read)
-				(*(wcb.callback_read))(wcb.fd, WATCH_READ, wcb.param_read);
+			{
+				#ifdef DEBUG_WATCH
+				fprintf(stderr, "call read callback on fd %d\n", wcb.fd);
+				#endif
+				(*(wcb.callback_read))(wcb.fd, GB_WATCH_READ, wcb.param_read);
+			}
 		}
 
-    if (FD_ISSET(wcb.fd, wfd))
+		if (watch_callback[i].fd < 0)
+			continue;
+		
+		if (FD_ISSET(wcb.fd, wfd))
 		{
+			FD_CLR(wcb.fd, wfd);
 			if (wcb.callback_write)
-				(*(wcb.callback_write))(wcb.fd, WATCH_WRITE, wcb.param_write);
+			{
+				#ifdef DEBUG_WATCH
+				fprintf(stderr, "call write callback on fd %d\n", wcb.fd);
+				#endif
+				(*(wcb.callback_write))(wcb.fd, GB_WATCH_WRITE, wcb.param_write);
+			}
 		}
-  }
+	}
 
 	_do_not_really_delete_callback--;
 	
 	if (!_do_not_really_delete_callback && _must_delete_callback)
 	{
+		#if DEBUG_WATCH
+		fprintf(stderr, "do must delete callback\n");
+		#endif
 		i = 0;
 		while (i < ARRAY_count(watch_callback))
 		{
@@ -453,46 +540,60 @@ static void raise_callback(fd_set *rfd, fd_set *wfd)
 
 static int do_select(fd_set *rfd, fd_set *wfd, struct timeval *timeout)
 {
-  int fd;
+	int fd;
 
-  for (fd = max_fd; fd >= 0; fd--)
-  {
-    if (FD_ISSET(fd, &read_fd) || FD_ISSET(fd, &write_fd))
-      break;
-  }
+	for (fd = max_fd; fd >= 0; fd--)
+	{
+		if (FD_ISSET(fd, &read_fd) || FD_ISSET(fd, &write_fd))
+			break;
+	}
 
-  if (fd < 0 && !timeout)
-    return 0;
+	if (fd < 0 && !timeout)
+		return 0;
 
-  max_fd = fd;
+	max_fd = fd;
 
-  *rfd = read_fd;
-  *wfd = write_fd;
+	*rfd = read_fd;
+	*wfd = write_fd;
 
-  return select(max_fd + 1, rfd, wfd, NULL, timeout);
+	return select(max_fd + 1, rfd, wfd, NULL, timeout);
 }
 
 static bool do_loop(struct timeval *wait)
 {
-  int ret;
-  struct timeval tv;
-  fd_set rfd, wfd;
-  bool something_done = FALSE;
+	int ret;
+	struct timeval tv;
+	fd_set rfd, wfd;
+	bool something_done = FALSE;
+
+	if (EVENT_check_post())
+		something_done = TRUE;
+
+	#ifdef DEBUG_TIMER
+	fprintf(stderr, "\ndo_loop: now = %.7g: select (%d)\n", time_to_double(time_now()), something_done);
+	#endif
 
 	if (get_timeout(wait, &tv))
 		ret = do_select(&rfd, &wfd, NULL);
 	else
 	{
 		ret = do_select(&rfd, &wfd, &tv);
-		something_done = TRUE;
+		something_done = ARRAY_count(_timers) > 0;
 	}
 
+	if (ret < 0 && errno != EINTR)
+		THROW_SYSTEM(errno, NULL);
+	
 	#ifdef DEBUG_TIMER
-	fprintf(stderr, "do_loop: now = %.7g\n", time_to_double(time_now()));
+	fprintf(stderr, "do_loop: now = %.7g: timers (%d)\n", time_to_double(time_now()), something_done);
 	#endif
 
 	raise_timers();
 
+	#ifdef DEBUG_TIMER
+	fprintf(stderr, "do_loop: now = %.7g: callbacks\n", time_to_double(time_now()));
+	#endif
+
 	if (ret > 0)
 	{
 		raise_callback(&rfd, &wfd);
@@ -500,13 +601,11 @@ static bool do_loop(struct timeval *wait)
 	}
 	else if (ret < 0)
 	{
-		if (errno != EINTR)
-			THROW_SYSTEM(errno, NULL);
 		something_done = TRUE;
 	}
 
-	if (EVENT_check_post())
-		something_done = TRUE;
+	//if (EVENT_check_post())
+	//	something_done = TRUE;
 
 	return something_done;
 }
@@ -520,14 +619,14 @@ bool WATCH_one_loop(int wait)
 		return do_loop(NULL);
 	else
 	{
-  	time_from_ms(&timeout, wait);
+		time_from_ms(&timeout, wait);
 		return do_loop(&timeout);
 	}
 }
 
 void WATCH_loop(void)
 {
-  while (do_loop(NULL));
+	while (do_loop(NULL));
 }
 
 
@@ -536,21 +635,21 @@ void WATCH_wait(int wait)
 	struct timeval *now;
 	struct timeval timeout;
 
-  if (wait == 0)
-  {
-  	timeout.tv_sec = 0;
-  	timeout.tv_usec = 0;
-  	do_loop(&timeout);
-  }
-  else
-  {
-  	now = time_now();
-  	time_from_ms(&timeout, wait);
-  	time_add(&timeout, now);
+	if (wait == 0)
+	{
+		timeout.tv_sec = 0;
+		timeout.tv_usec = 0;
+		do_loop(&timeout);
+	}
+	else
+	{
+		now = time_now();
+		time_from_ms(&timeout, wait);
+		time_add(&timeout, now);
 
 		for(;;)
 		{
-	  	now = time_now();
+			now = time_now();
 			if (time_lower_than(&timeout, now))
 				break;
 			do_loop(&timeout);
@@ -558,34 +657,51 @@ void WATCH_wait(int wait)
 	}
 }
 
-int WATCH_process(int fd_end, int fd_output)
+int WATCH_process(int fd_end, int fd_output, int fd_error, int timeout)
 {
-  fd_set rfd;
-  int ret, fd_max;
+	fd_set rfd;
+	int ret, fd_max;
+	struct timeval tv;
 
-  fd_max = fd_end > fd_output ? fd_end : fd_output;
+	fd_max = fd_end > fd_output ? fd_end : fd_output;
+	fd_max = fd_error > fd_max ? fd_error : fd_max;
 
-  for(;;)
-  {
-    FD_ZERO(&rfd);
-    FD_SET(fd_end, &rfd);
-    if (fd_output >= 0)
-      FD_SET(fd_output, &rfd);
+	for(;;)
+	{
+		FD_ZERO(&rfd);
+		FD_SET(fd_end, &rfd);
+		if (fd_output >= 0)
+			FD_SET(fd_output, &rfd);
+		if (fd_error >= 0)
+			FD_SET(fd_error, &rfd);
 
-    ret = select(fd_max + 1, &rfd, NULL, NULL, NULL);
+		if (timeout > 0)
+		{
+			tv.tv_sec = timeout / 1000;
+			tv.tv_usec = (timeout % 1000) * 1000;
+			ret = select(fd_max + 1, &rfd, NULL, NULL, &tv);
+			if (ret == 0)
+				break;
+		}
+		else
+		{
+			ret = select(fd_max + 1, &rfd, NULL, NULL, NULL);
+		}
 
-    if (ret > 0)
-      break;
-    if (errno != EINTR)
-      break;
-  }
+		//fprintf(stderr, "WATCH_process: select -> %d [%d] / fd_end -> %d\n", ret, errno, FD_ISSET(fd_end, &rfd));
+		
+		if (ret > 0)
+			break;
+		if (errno != EINTR)
+			break;
+	}
 
-  if (FD_ISSET(fd_end, &rfd))
-    return fd_end;
-  else if (FD_ISSET(fd_output, &rfd))
-    return fd_output;
-  else
-    return -1;
+	ret = timeout > 0 ? WP_TIMEOUT : WP_NOTHING;
+	
+	if (FD_ISSET(fd_end, &rfd)) ret += WP_END;
+	if (fd_output >= 0 && FD_ISSET(fd_output, &rfd)) ret += WP_OUTPUT;
+	if (fd_error >= 0 && FD_ISSET(fd_error, &rfd)) ret += WP_ERROR;
+	return ret;
 }
 
 
@@ -607,3 +723,17 @@ void WATCH_timer(void *t, int on)
 		timer->id = 0;
 	}
 }
+
+void WATCH_little_sleep(void)
+{
+	struct timeval tv;
+	struct timeval *now;
+
+	now = time_now();
+	tv = *now;
+	time_sub(&tv, &_last_time);
+	if (tv.tv_sec == 0 && tv.tv_usec < 10000)
+		usleep(10000 - tv.tv_usec);
+	_last_time = *now;
+}
+
