@@ -1,14 +1,12 @@
 /***************************************************************************
 
-  watch.h
+  gbx_watch.h
 
-  Default event loop and file descriptor watch routines
-
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -34,11 +33,13 @@
 #endif
 
 enum {
-  WATCH_NONE = 0,
-  WATCH_READ = 1,
-  WATCH_WRITE = 2,
-  };
-
+	WP_NOTHING = 0,
+	WP_END = 1,
+	WP_OUTPUT = 2,
+	WP_ERROR = 4,
+	WP_TIMEOUT = 8
+};
+	
 typedef
   struct {
     int fd;
@@ -56,7 +57,6 @@ typedef
 	}
 	WATCH_TIMER;
 
-
 void WATCH_init(void);
 void WATCH_exit(void);
 
@@ -65,7 +65,9 @@ bool WATCH_one_loop(int);
 void WATCH_loop(void);
 void WATCH_wait(int);
 int WATCH_loop_signal(const sigset_t *sig);
-int WATCH_process(int fd_end, int fd_output);
+int WATCH_process(int fd_end, int fd_output, int fd_error, int timeout);
 void WATCH_timer(void *t, int on);
+double WATCH_get_timeout(GB_TIMER *timer);
+void WATCH_little_sleep(void);
 
 #endif /* */
