@@ -2,11 +2,11 @@
 
   debug.h
 
-  (c) 2000-2007 Benoit Minisini <gambas@freesurf.fr>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -16,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -36,7 +37,6 @@ typedef
     PCODE *addr;
     CLASS *class;
     ushort line;
-		short _reserved;
     VALUE *bp;
     FUNCTION *fp;
     }
@@ -58,7 +58,6 @@ typedef
     DEBUG_TYPE type;
     void (*func)(const char *);
     bool loop;
-		char _reserved[3];
     }
   DEBUG_COMMAND;
 
@@ -75,19 +74,21 @@ EXTERN char DEBUG_buffer[];
 
 #define DEBUG_WELCOME   "Welcome to the Gambas debugger.\n"
 
-PUBLIC void DEBUG_breakpoint(int id);
-PUBLIC void DEBUG_main(boolean error);
+void DEBUG_breakpoint(int id);
+void DEBUG_main(bool error);
 
-PUBLIC const char *DEBUG_get_position(CLASS *cp, FUNCTION *fp, PCODE *pc);
-PUBLIC const char *DEBUG_get_current_position(void);
-PUBLIC void DEBUG_init_breakpoints(CLASS *class);
+bool DEBUG_calc_line_from_position(CLASS *class, FUNCTION *func, PCODE *addr, ushort *line);
+const char *DEBUG_get_position(CLASS *cp, FUNCTION *fp, PCODE *pc);
+const char *DEBUG_get_profile_position(CLASS *cp, FUNCTION *fp, PCODE *pc);
+const char *DEBUG_get_current_position(void);
+void DEBUG_init_breakpoints(CLASS *class);
 
-PUBLIC void DEBUG_break_on_next_line(void);
+void DEBUG_break_on_next_line(void);
 
-PUBLIC DEBUG_INFO *DEBUG_init(GB_DEBUG_INTERFACE *debug, bool fifo);
-PUBLIC void DEBUG_exit(void);
-PUBLIC void DEBUG_welcome(void);
-PUBLIC void DEBUG_where(void);
-PUBLIC void DEBUG_backtrace(FILE *out);
+DEBUG_INFO *DEBUG_init(GB_DEBUG_INTERFACE *debug, bool fifo, const char*fifo_name);
+void DEBUG_exit(void);
+void DEBUG_welcome(void);
+void DEBUG_where(void);
+void DEBUG_backtrace(FILE *out);
 
 #endif
