@@ -1,14 +1,12 @@
 /***************************************************************************
 
-  limit.h
+  gb_limit.h
 
-  Compiler and interpreter limits
-
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -44,7 +43,7 @@
 #define MAX_SYMBOL_LEN      255
 
 /* Maximum number of constants in the same class - CANNOT CHANGE */
-#define MAX_CLASS_CONST     4096
+#define MAX_CLASS_CONST     65536
 
 /* Maximum number of static or dynamic symbols in the same class - CANNOT CHANGE */
 #define MAX_CLASS_SYMBOL    2048
@@ -55,17 +54,17 @@
 /* Maximum number of class uses in the same class - CANNOT CHANGE */
 #define MAX_CLASS_CLASS     2048
 
-/* Maximum number of class uses in the same class - CANNOT CHANGE */
+/* Maximum number of extern declaration in the same class - CANNOT CHANGE */
 #define MAX_CLASS_EXTERN    256
 
 /* Maximum number of events in the same class - CANNOT CHANGE */
-#define MAX_CLASS_EVENT     256
+#define MAX_CLASS_EVENT     254
 
 /* Maximum number of unknown symbols in the same class - CANNOT CHANGE */
 #define MAX_CLASS_UNKNOWN   65536
 
 /* Maximum number of array declarations in the same class - CANNOT CHANGE */
-#define MAX_CLASS_ARRAY     32678
+#define MAX_CLASS_ARRAY     32768
 
 /* Maximum number of local variables in a function - CANNOT CHANGE */
 #define MAX_LOCAL_SYMBOL    127
@@ -76,9 +75,6 @@
 /* Maximum number of dimensions in an array */
 #define MAX_ARRAY_DIM       8
 
-/* Maximum length of a file path inside an archive */
-#define MAX_ARCH_PATH       512
-
 /* Maximum level of controls imbrication in a form */
 #define MAX_FORM_PARENT     32
 
@@ -97,5 +93,11 @@
 /* Maximum length of an error message */
 #define MAX_ERROR_MSG       511
 
+/* Maximum number of fields in a structure */
+#define MAX_STRUCT_FIELD    255
+
+/* Maximum number of digits in a Float */
+#define MAX_FLOAT_DIGIT     15
+
 #endif
 
