@@ -1,14 +1,12 @@
 /***************************************************************************
 
-  pcode.h
+  gb_pcode.h
 
-  The p-code instruction set
-
-  (c) 2000-2007 Benoit Minisini <gambas@freesurf.fr>
+  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -47,7 +46,7 @@
 #define C_PUSH_UNKNOWN          0x0400
 
 #define C_PUSH_EXTERN           0x0500
-//#define C_PUSH_SPECIAL          0x0600
+#define C_BYREF                 0x0600
 #define C_PUSH_EVENT            0x0700
 #define C_QUIT                  0x0800
 
@@ -69,29 +68,36 @@
 #define C_PUSH_MISC             0x1400
 #define C_PUSH_ME               0x1500
 
-#define CPM_NULL   0
-#define CPM_VOID   1
-#define CPM_FALSE  2
-#define CPM_TRUE   3
-#define CPM_LAST   4
+#define CPM_NULL         0
+#define CPM_VOID         1
+#define CPM_FALSE        2
+#define CPM_TRUE         3
+#define CPM_LAST         4
+#define CPM_STRING       5
+#define CPM_PINF         6
+#define CPM_MINF         7
+#define CPM_COMPLEX      8
+#define CPM_VARGS        9
+#define CPM_DROP_VARGS   10
+#define CPM_RETURN       11
 
-#define C_EVENT                 0x1600
+#define C_TRY                   0x1600
+#define C_END_TRY               0x1700
+#define C_CATCH                 0x1800
 
-#define C_TRY                   0x1700
-#define C_END_TRY               0x1800
-#define C_CATCH                 0x1900
+#define C_DUP                   0x1900
+#define C_DROP                  0x1A00
+#define C_NEW                   0x1B00
 
-#define C_DUP                   0x1A00
-#define C_DROP                  0x1B00
-#define C_NEW                   0x1C00
-
-#define C_CALL                  0x1D00
-#define C_CALL_QUICK            0x1E00
-#define C_CALL_NORM             0x1F00
+#define C_CALL                  0x1C00
+#define C_CALL_QUICK            0x1D00
+#define C_CALL_SLOW             0x1E00
+#define C_ON                    0x1F00
 
 #define C_JUMP                  0x2000
 #define C_JUMP_IF_TRUE          0x2100
 #define C_JUMP_IF_FALSE         0x2200
+#define C_GOSUB                 0x2300
 
 #define C_JUMP_FIRST            0x2400
 #define C_JUMP_NEXT             0x2500
@@ -128,8 +134,8 @@
 #define CODE_FIRST_SUBR 0x40
 #define CODE_LAST_SUBR  0x9F
 
-#define CODE_CALL_VARIANT   0x40
-#define CODE_CALL_VOID      0x80
+#define CODE_CALL_VARIANT   0x80
+//#define CODE_CALL_VOID      0x40
 
 #define CODE_STATIC    0x0800
 #define CODE_FUNCTION  0x0400
@@ -137,11 +143,14 @@
 #define CODE_NEW_ARRAY  0x40
 #define CODE_NEW_EVENT  0x80
 
+#define CODE_OCT         (CODE_FIRST_SUBR + 8)
 #define CODE_RINSTR      (CODE_FIRST_SUBR + 12)
 #define CODE_ABS         (CODE_FIRST_SUBR + 20)
 #define CODE_MAX         (CODE_FIRST_SUBR + 30)
 #define CODE_CONV        (CODE_FIRST_SUBR + 39)
-
+#define CODE_BIN         (CODE_FIRST_SUBR + 40)
+#define CODE_HEX         (CODE_FIRST_SUBR + 41)
+#define CODE_DEBUG       (CODE_FIRST_SUBR + 54)
 
 typedef
   ushort PCODE;
@@ -157,7 +166,7 @@ typedef
 #define PCODE_BREAKPOINT(num) ((PCODE)(C_BREAK | (num)))
 
 #ifndef NO_CODE_DUMP
-PUBLIC short PCODE_dump(FILE *out, short addr, PCODE *code);
+short PCODE_dump(FILE *out, ushort addr, PCODE *code);
 #endif
 
 #endif /* */
